const Ci               = Components.interfaces;
const Cc               = Components.classes;
var alexaUtilityService = null;
var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
var ld_prefs = prefs.getBranch("google.toolbar.linkdoctor.");
var alexa_prefs = prefs.getBranch("extensions.alexa.");
const MY_CID = "{9d27b588-161b-406b-adf8-c7a308b46efe}";
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);  
 
function trace(msg)
{
  Cc["@mozilla.org/consoleservice;1"]
    .getService(Ci.nsIConsoleService)
    .logStringMessage( "Alexa.errors: "
      + String( msg ) );
}

// If Herb's ping is greater than this, disable HERB
const HERB_MAX_LATENCY = 2000;

// If Herb's response time is greater than this, display regular error page  
const HERB_TIMEOUT = 3000;

var HerbMonitor = {
  'available': 0,
  'time': 0,
  'last_status': 200,
  'latency': 0,
  'ping': function(extra, callback)
  {
    var curTime = (new Date()).getTime();

    if (!HERB_PING_URI)
    {
      init_herb_base_uri();
    }
    if (!HERB_PING_URI)
    {
      return;
    }

    HerbMonitor.time = curTime;
    var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    var hitting_real_service = 0;
    var URI;
    var method = "HEAD";
    if (HerbMonitor.last_status != 200 && extra)
    {
      URI = HERB_BASE_URI + extra;
      hitting_real_service = 1
      method = "GET";
    }
    else
    {
      URI = HERB_PING_URI + extra;
    }
    if (HerbMonitor.latency)
    {
      URI += "&latency=" + HerbMonitor.latency;
      HerbMonitor.latency = 0;
    }
    http.open( method, URI, true );
    http.onload = function()
    {
      var newTime = (new Date()).getTime();
      HerbMonitor.latency = newTime - curTime;        
      if (http && HerbMonitor.latency <= HERB_MAX_LATENCY)
      {
        if (hitting_real_service)
        {
          HerbMonitor.last_status = http.status;
        }
        if (http.status == 200 || (http.status == 404 && !hitting_real_service))
        {
          HerbMonitor.available = 1;
          if (callback)
          {
            callback();
          }
        }
      }
      http = null;
    };
    http.send( null );
    var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    timer.initWithCallback({
      notify: function(timer)
      {
        if (http)
        {
          http.abort();
          http = null;
          HerbMonitor.latency = "timeout";
        }
      }
    }, 2000, Ci.nsITimer.TYPE_ONE_SHOT);
  }
};

function ensure_compatibility()
{
  if( alexa_prefs.getBoolPref( 'error-handling' ) )
  {
    // We're not compatible with the linkdoctor, so disable it.  
    ld_prefs.setBoolPref("enabled", false);
  }
}


var HERB_BASE_URI = 0;
var HERB_PING_URI = 0;

function GET_HERB_BASE_URI()
{
  return HERB_BASE_URI;
}

function QS2P( qs ) {
  if( qs == null ) return { };
  if( qs.indexOf( '?' ) >= 0 )
    qs = qs.split( '?' )[ 1 ];
  var query   = qs.split( '&' );
  var params  = { };
  var v;
  for( var i = 0; i < query.length; i++ ) {
    v = query[ i ].split( '=' );
    params[ v[ 0 ] ] = decodeURIComponent( v[ 1 ] || '' );
  }
   
  return params;
}

function init_herb_base_uri()
{
  var aid = "";
  if (alexa_prefs.prefHasUserValue('session'))
  {
    aid = alexa_prefs.getCharPref('session');
  }
  var ver = alexaUtilityService.getMyVersion( );
  var rnd = Math.random();

  HERB_BASE_URI = (
   'http://recovery.alexa.com/helper/'
   + '?aid='      + encodeURIComponent( aid )
   + '&plugin='   + encodeURIComponent( ver )
   + '&rnd=' + rnd
  );
  HERB_PING_URI = (
   'http://recovery.alexa.com/ping/'
   + '?aid='      + encodeURIComponent( aid )
   + '&plugin='   + encodeURIComponent( ver )
   + '&rnd=' + rnd
  );
}

function loadHERB(win, loc, referrer, extra)
{
  if (!HERB_BASE_URI)
  {
    init_herb_base_uri();
  }
  if (!HERB_BASE_URI)
  {
    return;
  }

  var saved = {
    "html": win.document.body.innerHTML,
    "win": win,
    "doc": win.document,
    "uri": win.document.documentURI
  };
  var URI = HERB_BASE_URI + extra;
  var mainWindow = getMainWindowForDocument(win.document);
  if (!mainWindow)
    return;
  var gBrowser = mainWindow.getBrowser();
  var browser = gBrowser.getBrowserForDocument(win.document);    
  function restore(timer)
  {
    if (saved.win.document == saved.doc && saved.uri == saved.win.document.documentURI)
    {
      HerbMonitor.latency = "timeout";
      HerbMonitor.available = 0;
      HerbMonitor.ping("", 0);

      saved.win.stop();
      if (saved.win.document == saved.doc && saved.uri == saved.win.document.documentURI)
      {
        saved.win.document.body.innerHTML = saved.html;
      }
      else
      {
        // Hmm, looks like we caught the page in mid-load. Refresh.
        saved.win.location.reload();
      }          
    }
    saved = null;
  }
  function goHERB()
  {
    var curTime = (new Date()).getTime();
    URI += "&time=" + curTime;
    if (HerbMonitor.latency)
    {
      URI += "&latency=" + encodeURIComponent(HerbMonitor.latency);
      HerbMonitor.latency = 0;
    }
    browser.loadURIWithFlags(URI, Ci.nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY, referrer);
    var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    timer.initWithCallback({
      notify: restore
    }, 3000, Ci.nsITimer.TYPE_ONE_SHOT);
  }

  if (!HerbMonitor.available || HerbMonitor.last_status != 200)
  {
    HerbMonitor.ping(extra, goHERB);
  }
  else
  {
    win.document.body.innerHTML = "";
    goHERB();
  }
}

function getMainWindowForDocument(doc)
{
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
             .getService(Components.interfaces.nsIWindowMediator);
  var mainWindow = wm.getMostRecentWindow("navigator:browser");
  var enumerator = null;
  var gBrowser = mainWindow.getBrowser();
  var idx = gBrowser.getBrowserIndexForDocument(doc);
  if (idx == -1)
    enumerator = wm.getEnumerator("navigator:browser");
  while (idx == -1)
  {
    if (!enumerator.hasMoreElements())
    {
      // Can't find any window for this document. Give up.
      return null;
    }
    mainWindow = enumerator.getNext();  
    gBrowser = mainWindow.getBrowser();     
    idx = gBrowser.getBrowserIndexForDocument(doc);
  }
  return mainWindow;
}

var onDomContent = function ( evt )
{
  if( !alexa_prefs.getBoolPref( 'error-handling' ) )
    return;
  var doc = null;
  try
  {
    doc = evt.originalTarget.QueryInterface(Ci.nsIDOMHTMLDocument);
  }
  catch (ex)
  {
  }
  if( doc != null )
  {
    var origuri = evt.originalTarget.documentURI;
    var win = evt.originalTarget.defaultView;
    ensure_compatibility();

    if (win && win.top == win && win.document && origuri)
    {
      var mainWindow = getMainWindowForDocument(win.document);
      if (!mainWindow)
        return;
      var gBrowser = mainWindow.getBrowser();
      var idx = gBrowser.getBrowserIndexForDocument(doc);
      if (idx == -1)
        return;
      var tab = gBrowser.mTabs[idx];
      var gURLBar = mainWindow.gURLBar;

      if (gURLBar && !gURLBar._HERB)
      {
        // ---
        // Hide new_tab_uri
        // NOTE: Be careful not to reference any variables
        //       other than this._HERB inside new_setter
        // ---
        var old_getter = gURLBar.__lookupGetter__("value");  
        gURLBar._HERB = {
          "old_setter": gURLBar.__lookupSetter__("value"),
          "GET_HERB_BASE_URI": GET_HERB_BASE_URI,
          "QS2P": QS2P,
          "trace": trace
        };
        function new_setter(value) {
          try
          {
            if (this._HERB.GET_HERB_BASE_URI() && value && value.indexOf(this._HERB.GET_HERB_BASE_URI()) == 0)
            {
              var params = this._HERB.QS2P(value);
              var origin_url = params["location"];
              if (origin_url)
              {
                return this._HERB.old_setter.call(this, origin_url);
              }
            }
          }
          catch (ex)
          {
            // Ignore exceptions
            this._HERB.trace(ex);
          }
        
          // Fall-through
          return this._HERB.old_setter.call(this, value);
        }
        gURLBar.__defineSetter__("value", new_setter);
        gURLBar.__defineGetter__("value", old_getter);
      }

      // NOTE: If the user has multiple toolbars installed, AlexaSkipURL may be modified
      // by multiple instances of HERB...
      var skipuri = sessionService.getTabValue(tab, "AlexaSkipUrl");
      if (skipuri == origuri)
      {
        // Looks like we've already processed this URL, so there's no need to
        // try this URL again.
        return;
      }
      else if (skipuri)
      {
        sessionService.setTabValue(tab, "AlexaSkipUrl", "");
      }

      if (HERB_BASE_URI && origuri.indexOf(HERB_BASE_URI) == 0)
      {
        var origuri_params = QS2P(origuri);
        var t = origuri_params["time"];
        var curTime = (new Date()).getTime();
        if (t && !HerbMonitor.latency)
        {
          HerbMonitor.latency = (curTime - t);
        }
      }
      else if( 0 == origuri.indexOf( 'about:neterror?e=' ) )
      {
        var origuri_params = QS2P(origuri);
        var e = origuri_params["e"];
        var u = origuri_params["u"];
        if (skipuri && skipuri == u)
        {
          // Looks like we just tried to go to recovery.alexa.com, and failed,
          // so there's no need to try this URL again.
          return;
        }
        else if (HERB_BASE_URI && u && u.indexOf(HERB_BASE_URI) == 0)
        {
          // It's an error about HERB, so disable HERB and go back to the original URL
          HerbMonitor.available = 0;
          HerbMonitor.latency = e;

          var params = QS2P(u);
          var origin_url = params["location"];
          var browser = gBrowser.getBrowserForDocument(win.document);
          if (origin_url)
          {
            win.document.body.innerHTML = "";
            sessionService.setTabValue(tab, "AlexaSkipUrl", origin_url);
            browser.loadURIWithFlags(origin_url, Ci.nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY);
          }
          return;
        }
        else if (u && u.indexOf("http://") == 0 && win.location.href != origuri)
        {
          var params = ['location=' + encodeURIComponent( u )];
          switch (e)
          {
            case "http404":
              e = "http";
              params.push('status=404');
            case "dnsNotFound":
            case "malformedURI":
            //case "netReset":
            //case "connectionFailure":
            //case "redirectLoop":
            //case "netTimeout":
            //case "netInterrupt":
              params.push('reason=' + encodeURIComponent( e ));
              loadHERB(win, u, null, '&' + params.join('&'));
            default:
              break;
          }
        }
      }

      var response = sessionService.getTabValue(tab, "ALX_NS_PH.response");
      if (response)
      {
        response = nativeJSON.decode(response);
      }
      if (response && response.URI == origuri && origuri.indexOf("http://") == 0)
      {
        var rfr = win.document.referrer;
        if (HERB_BASE_URI && rfr && rfr.indexOf(HERB_BASE_URI) == 0)
        {
          rfr = "";
        }

        if (HERB_BASE_URI && origuri.indexOf(HERB_BASE_URI) == 0)
        {
          HerbMonitor.last_status = response.responseStatus;
          if (response.responseStatus != 200)
          {
            var params = QS2P(origuri);
            var origin_url = params["location"];
            var browser = gBrowser.getBrowserForDocument(win.document);
            if (origin_url)
            {
              win.document.body.innerHTML = "";
              sessionService.setTabValue(tab, "AlexaSkipUrl", origin_url);
              browser.loadURIWithFlags(origin_url, Ci.nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY);
            }
          }
          return;
        }
        else if (response.responseStatus == "dnsNotFound"
            // || response.responseStatus == "connectionFailure"
            // || response.responseStatus == "malformedURI"
            // || response.responseStatus == "generic"
           )
        {
          var params = ('&reason=' + response.responseStatus
                      + '&location=' + encodeURIComponent( win.document.documentURI )
                      + '&referrer=' + encodeURIComponent( rfr ));
          loadHERB(win, win.document.documentURI, null, params);
        }
        else if (win.document.body && win.document.body.innerHTML.length < 512)
        {
          responseStatus = response.responseStatus;
          switch( responseStatus )
          {
              case 400:
              case 403:
              case 404:
              case 405:
              case 406:
              case 410:
              case 500:
              case 502:
              case 503:
              case 504:
                var params = ('&reason=http'
                              + '&location=' + encodeURIComponent( origuri )
                              + '&status='   + encodeURIComponent( responseStatus )
                              + '&referrer=' + encodeURIComponent( rfr )
                              + '&method='   + encodeURIComponent( response.requestMethod ));
                loadHERB(win, origuri, null, params);
              break;
            default:
              break;
          }
        }
      }
    }
  }
};

var sessionService = Cc["@mozilla.org/browser/sessionstore;1"].getService( Ci.nsISessionStore );

function myObserver()
{
}
myObserver.prototype = {
  QueryInterface: function (iid) {
    if (iid.equals(Ci.nsISupports) || iid.equals(Ci.nsIAlexaHerbService) || MY_CID.equals(iid))
        return this;

    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;

    return null;
  },
  observe: function(subject, topic, data)
  {
    if (topic == "toplevel-window-ready")
    {
      subject.addEventListener( "DOMContentLoaded", onDomContent, true );
    }    
    else if (topic == "http-on-examine-response" || topic == "http-on-examine-cached-response")
    {
      if (topic == "http-on-examine-response"  && !HerbMonitor.available && !HerbMonitor.time)
      {
        // Looks like we're connected to the network. Send out a ping.        
        HerbMonitor.ping("", 0);
      }
  
      var channel = subject.QueryInterface(Ci.nsIHttpChannel);
      var request = subject.QueryInterface(Ci.nsIRequest);
      if (!request.loadGroup)
        return;
      var loadGroup = request.loadGroup.QueryInterface(Ci.nsILoadGroup)
      if (!loadGroup.groupObserver)
        return;
      var webprog = loadGroup.groupObserver.QueryInterface(Ci.nsIWebProgress);
      if (!webprog)
        return;
      try
      {
        var win = webprog.DOMWindow;
      }
      catch (ex)
      { 
        return;
      }
      if (!webprog.isLoadingDocument || !win || win != win.top)
        return;      
      ensure_compatibility();
      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
      var mainWindow = getMainWindowForDocument(win.document);
      if (!mainWindow)
        return;
      var gBrowser = mainWindow.getBrowser();
      var idx = gBrowser.getBrowserIndexForDocument(win.document);
      if (idx == -1)
        return;
      var tab = gBrowser.mTabs[idx];
      var responseStatus = request.responseStatus;
      try
      {
        var err = request.getResponseHeader("X-Squid-Error");
        if (err)
        {
          if (/^ERR_DNS_FAIL/.test(err))
          {
            responseStatus = "dnsNotFound";
          }
          else if (/^ERR_INVALID_URL/.test(err))
          {
            responseStatus = "malformedURI";
          }
          else if (/^ERR_CONNECT_FAIL/.test(err))
          {
            responseStatus = "connectionFailure";
          }
          else
          {
            // Unknown problem
            responseStatus = "generic";
          }
        }
      }
      catch (ex)
      {
      }
      var d = {
        "responseStatus": responseStatus,
        "URI": request.URI.spec,
        "requestMethod": request.requestMethod,
      };
      var oldResponse = sessionService.getTabValue(tab, "ALX_NS_PH.response");
      if (oldResponse)
      {
        oldResponse = nativeJSON.decode(oldResponse);
        if (oldResponse && oldResponse.URI == win.document.documentURI)
        {
          sessionService.setTabValue(tab, "ALX_NS_PH.final-response", nativeJSON.encode(oldResponse));
        }
      }
      sessionService.setTabValue(tab, "ALX_NS_PH.response", nativeJSON.encode(d));
    }
  },
  getCurrentUrl: function(value)
  {
    if (HERB_BASE_URI && value.indexOf(HERB_BASE_URI) == 0)
    {
      var params = QS2P(value);
      var origin_url = params["location"];
      if (origin_url)
      {
        return origin_url;
      }
    }
    return value;
  }
}
var singleton = null;
var createInstanceFunc = function (outer, iid) {
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;
  
  if (!singleton)
  {
    alexaUtilityService = Cc["@ALX_NS_PH/utility-service;1"].createInstance(Ci.nsIAlexaUtilityService);
    singleton = (new myObserver());
    var observerService = Cc["@mozilla.org/observer-service;1"]
                                    .getService(Ci.nsIObserverService );
    observerService.addObserver(singleton, "http-on-examine-response", false);
    observerService.addObserver(singleton, "http-on-examine-cached-response", false);
    observerService.addObserver(singleton, "toplevel-window-ready", false);
  }

  return singleton.QueryInterface(iid);
};


var myModule = {
  firstTime: true,

  /*
   * RegisterSelf is called at registration time (component installation
   * or the only-until-release startup autoregistration) and is responsible
   * for notifying the component manager of all components implemented in
   * this module.  The fileSpec, location and type parameters are mostly
   * opaque, and should be passed on to the registerComponent call
   * unmolested.
   */
  registerSelf: function (compMgr, fileSpec, location, type) {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID,
                                    "HERB",
                                    this.myProgID,
                                    fileSpec,
                                    location,
                                    type);
    var catman = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
    catman.addCategoryEntry("app-startup", "HERB", this.myProgID, true, true);
  },

  unregisterSelf: function(compMgr, fileSpec, location) {
    var compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
    compMgr.unregisterFactoryLocation(this.myCID, fileSpec);
    var catman = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
    catman.deleteCategoryEntry("app-startup", this.myProgID, true);
  },

  /*
   * The GetClassObject method is responsible for producing Factory objects
   */
  getClassObject: function (compMgr, cid, iid) {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    if (!iid.equals(Ci.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.myFactory;
  },

  /* CID for this class */
  myCID: Components.ID(MY_CID),

  /* ProgID for this class */
  myProgID: "@ALX_NS_PH/herb;1",

  /* factory object */
  myFactory: {
    /*
     * Construct an instance of the interface specified by iid, possibly
     * aggregating it with the provided outer.  (If you don't know what
     * aggregation is all about, you don't need to.  It reduces even the
     * mightiest of XPCOM warriors to snivelling cowards.)
     */
    createInstance: createInstanceFunc
  },
  
  createInstance: createInstanceFunc,

  /*
   * The canUnload method signals that the component is about to be unloaded.
   * C++ components can return false to indicate that they don't wish to be
   * unloaded, but the return value from JS components' canUnload is ignored:
   * mark-and-sweep will keep everything around until it's no longer in use,
   * making unconditional ``unload'' safe.
   *
   * You still need to provide a (likely useless) canUnload method, though:
   * it's part of the nsIModule interface contract, and the JS loader _will_
   * call it.
   */
  canUnload: function(compMgr) {
      return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
    return myModule;
}

function NSGetFactory(cid) {
    if (cid.toString().toUpperCase() != MY_CID.toString().toUpperCase()) throw Components.results.NS_ERROR_FACTORY_NOT_REGISTERED;
    return myModule;
}

