/******************
 * Liangnan Wu *
 * Alexa Internet *
 ******************/

(function() {
  const Ci = Components.interfaces;
  const Cc = Components.classes;
  const sessionService = Cc["@mozilla.org/browser/sessionstore;1"].getService( Ci.nsISessionStore );
  const nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
  const ioService = Cc["@mozilla.org/network/io-service;1"]
                     .getService(Ci.nsIIOService);
  const promptSvc = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                     .getService(Ci.nsIPromptService);
  const appSvc = Cc["@mozilla.org/appshell/appShellService;1"]
                  .getService(Ci.nsIAppShellService);
  const environment = Cc["@mozilla.org/process/environment;1"]
                  .getService(Ci.nsIEnvironment);
  const verSvc = Cc["@mozilla.org/xpcom/version-comparator;1"]
                  .getService(Ci.nsIVersionComparator);
  const htmlSrc = Cc["@mozilla.org/feed-unescapehtml;1"]
          .getService(Components.interfaces.nsIScriptableUnescapeHTML)
  const bundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
          .getService(Components.interfaces.nsIStringBundleService);
  const categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
      .getService(Components.interfaces.nsICategoryManager);

  var init_in_progress = false;
  var configure_window = null;
  var about_window      = null;
  var option_window     = null;
  var locale_window     = null;
  var popup_panels = {};
  var feedwrappers = {};
  var iframes = {};
  var logo = null;
  var observers = [];
  var lastAmazonpopup = null;
  var searchboxes = { "_text": {
    "searchingText": "Searching...",
    "searchSuggestionText": "Search Suggestion...",
    "notAvailableText": "No Result",
    "loadingErrorText": "Loading Error..."
  } };  

  var loadContentHelp = {
    searchTimeout: 600,
    searchCurrentTime: 0,
    currentSearchWord: null,
    currentLocale: null,
    stringBundle: null,
    isJsorXml: function(fileName) 
    {
      if (typeof fileName != "string")
        return false;
      
      if (fileName.substr(-4) == ".xml" || fileName.substr(-3) == ".js")
        return true;
      return false;
    },
    loadLocale: function()
    {
      if (!loadContentHelp.stringBundle)
      {
        var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
        loadContentHelp.stringBundle = {};
        loadContentHelp.currentLocale = "default";
        if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocale") )
          loadContentHelp.currentLocale = prefBranch.getCharPref("extensions.alexa.toolbarLocale");
          
        if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocaleText") )
        {
          var toolbarLocaleText = decodeURIComponent(escape(prefBranch.getCharPref("extensions.alexa.toolbarLocaleText")));
          var dp = new DOMParser();
          var doc = dp.parseFromString(toolbarLocaleText, "text/xml");
          var buttons = doc.documentElement.getElementsByTagName("button");
          for ( var i = 0 ; i < buttons.length ; ++i )
          { 
            var button = buttons.item(i);
            var id = button.getAttribute("id");
            loadContentHelp.stringBundle[id] = {"visible": true, "properties": {}};
            var children = button.childNodes;
            for (var j = 0; j < children.length; j++)
            {
              var item = children.item(j);
              var tagName = item.tagName;
              switch (tagName)
              {
                case "localeReplacement":
                  var lid  = item.getAttribute("id");
                  loadContentHelp.stringBundle[id]["properties"][lid] = item.textContent;
                  break;
                case "visible":
                  var value = item.textContent;
                  if (value.replace(/\s/g, "") == "false")
                    loadContentHelp.stringBundle[id]["visible"] = false;
                  break;
              }
            }
          }

          var configureNodes = doc.documentElement.getElementsByTagName("configuration");
          loadContentHelp.stringBundle["_configuration"] = {"properties": {}};
          for ( var i = 0 ; i < configureNodes.length ; ++i )
          {
            var configureNode = configureNodes.item(i);
            var children = configureNode.childNodes;
            for (var j = 0; j < children.length; j++)
            {
              var item = children.item(j);
              var tagName = item.tagName;
              switch (tagName)
              {
                case "localeReplacement":
                  var lid  = item.getAttribute("id");
                  loadContentHelp.stringBundle["_configuration"]["properties"][lid] = item.textContent;
                  break;
              }
            }
          }
          
          var replacementsNode = doc.documentElement.getElementsByTagName("replacements")[0];
          if (replacementsNode)
          {
            var replacementNodes = replacementsNode.getElementsByTagName("replacement");
            loadContentHelp.stringBundle["_replacement"] = {"properties": {}};
            for ( var i = 0 ; i < replacementNodes.length ; ++i )
            {
              var replacementNode = replacementNodes.item(i);
              var keyNodes        = replacementNode.getElementsByTagName("key");
              var valueNodes      = replacementNode.getElementsByTagName("value");
              
              for (var j = 0; j < keyNodes.length; j++)
              {
                var key   = keyNodes[j].textContent;
                var value = valueNodes[j].textContent
                loadContentHelp.stringBundle["_replacement"]["properties"][key] = value;
              }
            }
            
          }
        }
      }
    },
    getLocalNameConfigure: function(defaultValue, propertyId)
    {
      loadContentHelp.loadLocale();
      if ( !propertyId || !loadContentHelp.stringBundle)
        return defaultValue;  
      
      var tmp = null;
      if (loadContentHelp.stringBundle["_configuration"])
        try {
          if (typeof loadContentHelp.stringBundle["_configuration"]["properties"][propertyId] != "undefined")
            tmp = loadContentHelp.stringBundle["_configuration"]["properties"][propertyId];
        } catch (e) {}

      if (loadContentHelp.stringBundle["_replacement"])
        try {
          if (typeof loadContentHelp.stringBundle["_replacement"]["properties"][propertyId] != "undefined")
            tmp = loadContentHelp.stringBundle["_replacement"]["properties"][propertyId];
        } catch (e) {}

      if(tmp == null)
        return defaultValue;
      else
        return tmp;
    },
    getLocalName: function(defaultValue, buttonId, propertyId)
    {
      //loadContentHelp.getLocalName
      if (!buttonId || !propertyId || !loadContentHelp.stringBundle)
        return defaultValue;
      
      var tmp = null;
      if (loadContentHelp.stringBundle[buttonId])
        try {
          tmp = loadContentHelp.stringBundle[buttonId]["properties"][propertyId];
        } catch (e) {}
      
      if(tmp == null)
        return defaultValue;
      else
        return tmp;
    },
    getTime: function()
    {
      var t = new Date();
      return t.getTime();
    },
    activeMenuItem: function(popup, i)
    {
      var tmp = popup;
      if (typeof popup == "string")
        tmp = document.getElementById(popup);
      var length = tmp.childNodes.length;

      if (i["index"] < 0)
      {
        i["index"] = -1;
        i["index"] = i["index"] + length;
      }
      if (i["index"] >= length)
      {
        i["index"] = i["index"] - length;
      }

      for (var tempNum = 0; tempNum < length; tempNum++) {
        var child = tmp.childNodes[tempNum];
        if (tempNum === i["index"]) {
          child.setAttribute('_moz-menuactive', "true");
        } else
          child.setAttribute('_moz-menuactive', "false");
      }
    },
    numPopup : 0,
    HTMLParser: function(aHTMLString){
       var parseDOM = content.document.createElement('div');
       parseDOM.appendChild(htmlSrc.parseFragment(aHTMLString, false, null, parseDOM));
       return parseDOM;
    },
    onDefaultText: function(child)
    { // loadContentHelp.onDefaultText()
      child.style.color = "gray";
      child.style.fontStyle = "italic";
    },
    offDefaultText: function(child)
    { // loadContentHelp.offDefaultText()
      child.style.color = "";
      child.style.fontStyle = "";
    },
    getRightColor: function(item)
    { // loadContentHelp.getRightColor
      item.style.color = "";
      var color = window.getComputedStyle(item,null).getPropertyValue("color");
      color = loadContentHelp.parseColor(color);

      if (color)
      {
        var sum = ( color[0] + color[1] + color[2] ) / 3;
        if (sum > 127.5)
          item.style.color = "#D5FFFF";
        else
          item.style.color = "blue";
      }
    },
    parseColor: function(color)
    { // loadContentHelp.parse
      if (typeof color == "string")
      {
        var tmp = color.split(')')[0];
        tmp = tmp.split('rgb(');
        if (tmp.length > 0)
        {
          tmp = tmp[1];
          tmp = tmp.split(", ");
          //trace(tmp)
          for (var i = 0; i < tmp.length; ++i)
            tmp[i] = parseInt(tmp[i]);
          return tmp;
        }
      }
      return null;
    },
    downloadFile: function(httpLoc, filePath) {
      try {     //loadContentHelp.downloadFile();
        var obj_URI = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(httpLoc, null, null);
        var obj_TargetFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

        obj_TargetFile.initWithPath(filePath);
        if(!obj_TargetFile.exists()) {
          obj_TargetFile.create(0x00,0644);
        }
        var obj_Persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);

        const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
        const flags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
        obj_Persist.persistFlags = flags | nsIWBP.PERSIST_FLAGS_FROM_CACHE;

        var downloadListener = function( afile )
        {
          this.targetFile = afile;
        };
        downloadListener.prototype = {
          targetFile: null,
          QueryInterface: function(aIID) {
            if (aIID.equals(Components.interfaces.nsIWebProgressListener)   ||
                aIID.equals(Components.interfaces.nsIWebProgressListener2)  ||
                aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
                aIID.equals(Components.interfaces.nsISupports))
              return this;
            throw Components.results.NS_NOINTERFACE;
          },

          onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus)
          {
            if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP)
            {
              var httpChannel = aRequest.QueryInterface(Ci.nsIHttpChannel);
              var statusCode = httpChannel.responseStatus;

              if (statusCode >= 400 && this.targetFile)
                this.targetFile.remove(false);
            }
          },
          onLocationChange: function(aProgress, aRequest, aURI) {},
          onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) { },
          onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) { },
          onSecurityChange: function() { },//ALX_NS_PH_SPARKLINE.trace("onSecurityChange"); },
          onProgressChange64: function() { },//ALX_NS_PH_SPARKLINE.trace("onProgressChange64"); },
          onRefreshAttempted: function() { return true; }//ALX_NS_PH_SPARKLINE.trace("onRefreshAttempted"); return true; }
        }

        obj_Persist.progressListener = new downloadListener(obj_TargetFile);
        try {
          obj_Persist.saveURI(obj_URI,null,null,null,null,obj_TargetFile);
        } catch(e)
        {
          obj_TargetFile.remove(false);
        }
      } catch (e) {
        trace(e);
      }
    },
    validContentUrl: function(url)
    {
      var validUrls = [ "https://s3.amazonaws.com/com.alexa.toolbar/",
                              "http://s3.amazonaws.com/com.alexa.toolbar/",
                              "https://tbar.alexa.com/",
                              "http://tbar.alexa.com/",
                              "chrome://ALX_NS_PH/"]

      for (var i = 0; i < validUrls.length; ++i)
      {
        var validUrl = validUrls[i];
        if (url.indexOf(validUrl)  == 0)
          return true;
      }
      return false;
    },
    parseOneXmlItem: function(menuItemChildren)
    {
      //var menuItemChildren = oneNode.childNodes;
      var menuItemProperties = {haschild: false, childs: null};
      var menuItemNodes = [];
      for (var j = 0; j < menuItemChildren.length; j++)
      {
        var item = menuItemChildren.item(j);
        var tagName = item.tagName;
        switch (tagName)
        {
          case "type":
          case "displayName":
          case "url":
          case "suggestionUrl":
          case "defaultText":
          case "openIn":
          case "image":
          case "icon":
          case "height":
          case "width":
          case "windowFeatures":
            menuItemProperties[tagName] = item.textContent;
            break;
          case "feedUrl":
            menuItemProperties["url"] = item.textContent;
            break;
          case "menuItems":
            menuItemNodes = item.childNodes;
            break;
          default:
            break;
        }
      }

      //try {
      if (menuItemProperties.type == "subMenu")
      {
        menuItemProperties.haschild = true;
        menuItemProperties.childs = []
        for (var i = 0; i < menuItemNodes.length; ++i)
        {
          var node = menuItemNodes.item(i);
          var pNode = loadContentHelp.parseOneXmlItem(node.childNodes);
          if (pNode.type)
            menuItemProperties.childs.push(pNode);
        }
      }//} catch(e) {alert(e)}
      return menuItemProperties;
    },
    parseOneXmlMenu: function(oneNode)
    {
      if (oneNode.tagName == "menuItem")
        return loadContentHelp.parseOneXmlItem(oneNode.childNodes);

      if (oneNode.tagName == "menuItems")
      {
        var menuChildren = oneNode.childNodes;
        var menuProperties = {type: "submenu", childs: []};
        for (var j = 0; j < menuChildren.length; j++)
        {
          var item = menuChildren.item(j);
          if (item.tagName == "displayName")
            menuProperties["displayName"] = item.textContent;
          if (item.tagName == "menuItem" || item.tagName == "menuItems")
            menuProperties.childs.push(loadContentHelp.parseOneXmlMenu(item));
        }

        return menuProperties;
      }
    },
    loadXmlMenuToProperty: function(menuItemNodes)
    {
      var menuItems = [];
      for (var i = 0; menuItemNodes && i < menuItemNodes.length; i++)
      {
        if (menuItemNodes.item(i).tagName != "menuItem")
          continue;
        menuItems.push(loadContentHelp.parseOneXmlItem(menuItemNodes.item(i).childNodes));
      }
      return menuItems;
    },
    loadFeedsFromUrl: function(properties, callback)
    {
      if (!properties.feedUrlUrl)
        return
        
      var url = ALX_NS_PH_TB.checkLocalDefault(properties.feedUrlUrl);
      var http = new XMLHttpRequest();
      http.open( "GET", url, true );
      http.onload = function()
      {

        var xml = http.responseXML;
        var menuItemNodes = [];
        if (xml.firstChild.tagName == "menuItems")
          menuItemNodes = xml.firstChild.childNodes;
        else
          menuItemNodes = xml.childNodes;

        var itemList = loadContentHelp.loadXmlMenuToProperty(menuItemNodes);
        
        for (var i in itemList)
        {
          var item = itemList[i];
          if (item.url)
            if (properties.feedUrls)
              properties.feedUrls.push(item.url);
            else
              properties.feedUrls = [item.url];
        }

        callback();
      };
      try {
        http.send(null);
      } catch(e) {}
      
    },
    loadMenuFromUrl: function(parent, properties)
    {
      if (!properties.menuUrl)
        return
        
      var url = ALX_NS_PH_TB.checkLocalDefault(properties.menuUrl);
      var http = new XMLHttpRequest();
      http.open( "GET", url, true );
      http.onload = function()
      {
        //try {
          var xml = http.responseXML;
          var menuItemNodes = [];
          if (xml.firstChild.tagName == "menuItems")
            menuItemNodes = xml.firstChild.childNodes;
          else
            menuItemNodes = xml.childNodes;

          var itemList = loadContentHelp.loadXmlMenuToProperty(menuItemNodes);
          //trace("itemlist" + itemList[0].displayName);
          //trace("tagName:" + menuItemNodes);
          //itemList = eval( '(' + text + ')' );
          //if (itemList.menuItems)
          //  itemList = itemList.menuItems;

          loadContentHelp.loadMenu(parent, properties, itemList);
       //} catch (e) {trace(e)}
      };
      try {
        http.send(null);
      } catch(e) {}
    },
    addSepar: function(toolbaritem)
    {
      var id = toolbaritem.getAttribute("id");
      if (!id)
          return;

      if (!toolbaritem.nextSibling || toolbaritem.nextSibling.tagName != "toolbarseparator")
      {
        var spacer = document.createElement("toolbarseparator");
        spacer.setAttribute("id", id + "-1");
        spacer.style.margin = "3px";
        toolbaritem.parentNode.insertBefore(spacer, toolbaritem.nextSibling);
      }
      if (!toolbaritem.previousSibling || toolbaritem.previousSibling.tagName != "toolbarseparator")
      {
        var spacer = document.createElement("toolbarseparator");
        spacer.setAttribute("id", id + "-2");
        spacer.style.margin = "3px";
        toolbaritem.parentNode.insertBefore(spacer, toolbaritem);
      }
    },
    loadMenu: function(parent, properties, menuItems, isforce)
    {
      if (isforce)
      {
        while (parent.firstChild) {
          parent.removeChild(parent.firstChild);
        }
      } else {
        if (parent.firstChild)
          return;
      }
      var id = properties.id;
      var image = null;
      var icon  = false;
      if (properties.menuImage) image = properties.menuImage;
      if (properties.menuIcon) icon = properties.menuIcon;
      
      var menupopup = document.createElement("menupopup");
      menupopup.setAttribute("onpopupshowing", "ALX_NS_PH_TB.log(" + sq(id) + " + ':m');");
      parent.appendChild(menupopup);
      
      for (var i = 0; i < menuItems.length; i++)    
      {
        let menuitem = null;
        var menuItemProperties = menuItems[i];
        let url = menuItemProperties.url,
            menuType = menuItemProperties.type,
            openIn = "currentTab",
            windowFeatures = "";
            args = null;
        if (menuType == "backgroundOptions")
          menuType = "options";
        if (menuItemProperties.type == "backgroundOptions" || menuItemProperties.type == "options" || menuItemProperties.type == "locale")
        {
          //url = "__TOOLBAR__ROOT__PLACEHOLDER__/../content/options.xul";
          if (menuItemProperties.type == "options" || menuItemProperties.type == "backgroundOptions")
            url = "__TOOLBAR__LOCALE__BASE__PLACEHOLDER__/" + loadContentHelp.currentLocale + "/bubbles/options.htm";
          else if (menuItemProperties.type == "locale")
            url = "__TOOLBAR__LOCALE__BASE__PLACEHOLDER__/" + loadContentHelp.currentLocale + "/bubbles/locale.htm";
          openIn = "newDialog";
          //windowFeatures = "centerscreen,close";
          if (menuItemProperties.width && menuItemProperties.height)
            windowFeatures = "alexaapi,centerscreen,resizable=no,status=no,height=" + menuItemProperties.height + ",width=" + menuItemProperties.width;
          else
            windowFeatures = "alexaapi,centerscreen,resizable=no,status=no,height=230,width=396";
        }
        else if (menuItemProperties.type == "organize")
        {
          url = "__TOOLBAR__ROOT__PLACEHOLDER__/../content/buttons.xul";
          openIn = "newDialog";
          windowFeatures = "centerscreen,resizable,close";
        }
        else if (menuItemProperties.type == "about")
        {
          //url = "__TOOLBAR__ROOT__PLACEHOLDER__/../content/about.xul";
          url = "__TOOLBAR__LOCALE__BASE__PLACEHOLDER__/" + loadContentHelp.currentLocale + "/bubbles/about.htm";
          openIn = "newDialog";
          //windowFeatures = "centerscreen";
          if (menuItemProperties.width && menuItemProperties.height)
            windowFeatures = "centerscreen,resizable=no,modal,status=no,height=" + menuItemProperties.height + ",width=" + menuItemProperties.width;
          else
            windowFeatures = "centerscreen,resizable=no,modal,status=no,height=176,width=330";
        }

        if (menuItemProperties.type == "menuBreak")
        {
          var menuseparator = document.createElement("menuseparator");
          menupopup.appendChild(menuseparator);
        }
        else
        {
          if (menuItemProperties.haschild || menuItemProperties.type == "rss")
            menuitem = document.createElement("menu");
          else
            menuitem = document.createElement("menuitem");
          if (menuItemProperties.openIn)
            openIn = menuItemProperties.openIn;
          if (menuItemProperties.windowFeatures)
            windowFeatures = menuItemProperties.windowFeatures;
          if (image)
            menuItemProperties.image = image;
          if (icon)
            menuItemProperties.icon = icon;
          menuitem.setAttribute("tooltiptext", menuItemProperties.displayName);
          menuitem.setAttribute("label", menuItemProperties.displayName);
          if (menuItemProperties.image)
            menuitem.setAttribute("image", ALX_NS_PH_TB.checkLocalDefault(menuItemProperties.image));
          if (menuItemProperties.icon)
          {
              menuitem.setAttribute("class","menuitem-iconic menu-iconic alexa-favicon");
          }
          if (!menuItemProperties.haschild && url && menuItemProperties.type != "rss")
          {
            menuitem.oncommand = function(event)
            {
              ALX_NS_PH_TB.log(id + ':i');
              if ( menuType == "locale" )
              {
                if ( locale_window == null || locale_window.closed )
                  locale_window = ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                else if (locale_window)
                  locale_window.focus();
                else
                  ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                event.stopPropagation();
              } 
              else if ( menuType == "about" )
              {
                if ( about_window == null || about_window.closed )
                  about_window = ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                else if (about_window)
                  about_window.focus();
                else
                  ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                event.stopPropagation();
              }
              else if ( menuType == "backgroundOptions" || menuType == "options"   )
              {
                if (option_window == null || option_window.closed)
                  option_window = ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                else if (option_window)
                  option_window.focus();
                else
                  ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                event.stopPropagation();
              } 
              else {
                ALX_NS_PH_TB.internalLoadURL(url, openIn, windowFeatures);
                event.stopPropagation();
              }
            };
            menuitem.addEventListener("command", menuitem.oncommand, false);
          }
          
          if (menuItemProperties.type == "backgroundOptions" || menuItemProperties.type == "options" || menuItemProperties.type == "about")
          {
            var branch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
            var observer = {
              observe: function(subject, topic, data)
              {
                if (topic == "nsPref:changed")
                {
                  menuitem.oncommand();
                }
              },
              shutdown: function()
              {
                branch.removeObserver("extensions.alexa.bubbles." + menuType, this);
              }
            };
            branch.addObserver("extensions.alexa.bubbles." + menuType, observer, false);
            observers.push(observer);
          }

          if (!menuItemProperties.haschild && menuItemProperties.type == "rss")
          {
            var menupopup1 = document.createElement("menupopup");
            let popupid= "ALX_NS_PH_TB-Button-" + id + "-popup-" + menuItemProperties.displayName  + "-" + loadContentHelp.numPopup;

            menupopup1.setAttribute("id", popupid);
            loadContentHelp.numPopup += 1;
            url = subst_tokens(url);
            loadContentHelp.requestAndAppendPopup(properties, menuitem, menupopup1, url, false, null);
            menupopup1.addEventListener("popupshowing", menupopup1.onpopupshowing, false);
            menuitem.appendChild(menupopup1);
          }
          if (menuItemProperties.type == "backgroundOptions")
            menuitem.style.display = "none";
          menupopup.appendChild(menuitem);
        }
        
        if (menuItemProperties.haschild)
        {
          //trace("haschild:" + menuItemProperties.haschild);
          //trace("childs:" + menuItemProperties.childs);
          loadContentHelp.loadMenu(menuitem, properties, menuItemProperties.childs, isforce);
        }
      }
    },
    requestAndAppendPopup: function(property, parent, popup, url, isCommandOrPopup, handle)
    {
      var popupid = popup.getAttribute("id");
      if (!popupid)
        popupid = property.id;
      var tmp = null;
  
      if (handle)
        tmp = handle;
      else
      {
        tmp = function(event)
        {
          ALX_NS_PH_TB.log(popupid + ':m');
          loadContentHelp.showpopuphelp(parent, popupid, searchboxes["_text"]["searchingText"]);
          lastAmazonpopup = popupid;
          var feedLoader = new FeedLoader({
            handleResult: function(result) {
              var feed = result.doc.QueryInterface(Ci.nsIFeed);

              if (!feed)
              {
                loadContentHelp.showpopuphelp(parent, popupid, searchboxes["_text"]["loadingErrorText"]);
                return;
              }
              // read the item and hash it

              var itemList = [];
              for (var i = 0; i < feed.items.length && i < 10; i++)
              {
                var item = feed.items.queryElementAt(i, Ci.nsIFeedEntry);
                var title = item.title.plainText();
                var link = item.link;
                loadContentHelp.HTMLParser(item.summary.text)
                var imgsrc = null;
                var desc   = null;
                var star   = null;
                var strike = null;
                var type   = null;
                try {
                  var divdoc = loadContentHelp.HTMLParser(item.summary.text);
                  imgsrc = divdoc.getElementsByTagName("img")[0].src;
                  //alert(property.id)
                  if (property.id == "bestsellers")
                  {
                    type = "amazon";
                    desc = divdoc.getElementsByTagName("font")[1].textContent;
                  }
                  else if (property.id == "newreleases")
                  {
                    type = "amazon";
                    desc = divdoc.getElementsByTagName("font")[0].textContent;
                  }
                  else if (property.id == "deals")
                  {
                    type = "amazon";
                    desc = divdoc.getElementsByTagName("td")[3].textContent;
                    if (desc.indexOf("Deal Price:") != -1)
                    {
                      desc = desc.replace(/Deal Price:/g, "");
                      desc = desc.replace(/^\s/g, "");
                    }
                    else
                      desc = null;
                  }
                    
                  
                  for(var j = 0; j < divdoc.getElementsByTagName("img").length; ++j)
                  {
                    var img = divdoc.getElementsByTagName("img")[j];
                    var alt = img.getAttribute("alt");
                    if (alt.indexOf("stars") >= 0)
                      star = img.src;
                  }
                  strike = divdoc.getElementsByTagName("strike")[0].textContent;
                }catch(e) {trace(e)}

                if (title == "")
                  title = item.summary.text;

                var itemLink = link.spec;
                if (property.urlRegexTo && property.urlRegexFrom)
                {
                  var reg = new RegExp(property.urlRegexFrom, "gi");
                  if ( itemLink.search( reg ) >= 0)
                    itemLink = itemLink.replace( reg, property.urlRegexTo);
                  else
                    itemLink = itemLink + property.urlRegexTo;
                }
                itemList.push({"text" : title, "url": itemLink, "image": imgsrc, "desc": desc, "star": star, "strike": strike, "type": type, "defaultText": property.noPriceAvailable});
              }

              if (lastAmazonpopup == popupid) 
              {
                if ( itemList.length > 0 )
                  loadContentHelp.renderResult(parent, popupid, itemList);
                else
                  loadContentHelp.showpopuphelp(parent, popupid, searchboxes["_text"]["notAvailableText"]);
              }
            },
            handleError: function() { loadContentHelp.showpopuphelp(parent, popupid, searchboxes["_text"]["loadingErrorText"]); }
          });
          feedLoader.load( url );
          event.stopPropagation();
        };
      }
      
      if (isCommandOrPopup)
        parent.oncommand = tmp;
      else
        popup.onpopupshowing = tmp; 
    },
    showpopuphelp: function(parent, id, message)
    {
      var popup = document.getElementById(id);
      
      while (popup.firstChild) {
        popup.removeChild(popup.firstChild);
      }

      popup.setAttribute("style", "background-color: menu;");
      var menuitem  = document.createElement("menuitem");
      menuitem.setAttribute("tooltiptext", message);
      var hbox      = document.createElement("hbox");
      hbox.setAttribute("style", "padding-top: 3px; padding-bottom: 3px;");
      var vbox      = document.createElement("vbox");
      var label     = document.createElement("label");
      label.setAttribute("value", message);
      label.setAttribute("class", "graymenu");
      vbox.appendChild(label);
      hbox.appendChild(vbox);
      menuitem.appendChild(hbox);
      popup.appendChild(menuitem);
      
      popup.openPopup(parent, "after_start", 0, 0, true, false);
    },
    
    renderResult: function(parent, id, itemlist, shown)
    { 
      var popup = document.getElementById(id);
      while (popup.firstChild) {
        popup.removeChild(popup.firstChild);
      }
      
      for (var i = 0; i < itemlist.length; ++i)
      {
        let item = itemlist[i];
        var menuitem  = document.createElement("menuitem");
        menuitem.setAttribute("tooltiptext", item.text);
        menuitem.setAttribute("maxwidth", "350px");
        menuitem.setAttribute("minwidth", "150px");
        var hbox      = document.createElement("hbox");
        hbox.setAttribute("style", "padding-top: 3px; padding-bottom: 3px;");
        
        var vbox1      = document.createElement("vbox");
        vbox1.setAttribute("style", "margin-left: 25px;");
        if (item.image) {
          vbox1.setAttribute("style", "margin-left: 30px; margin-right: 5px;");
          vbox1.setAttribute("width", "48px");
          vbox1.setAttribute("height", "48px");
          vbox1.setAttribute("align", "center");
        }

        var image1     = document.createElement("image"); 
        if (item.image)
        {
          image1.setAttribute("src", item.image);
          image1.setAttribute("width", "48px");
          image1.setAttribute("height", "48px");
        }

        if (!item.desc && "amazon" == item.type)
        {
          if (item.defaultText)
            item.desc = item.defaultText;
          else
            item.desc = "See Price on Amazon";
        }
        else
          item.desc = item.desc.replace(/^\s+/g, "");
        
        var vbox2      = document.createElement("vbox");
        var label1     = document.createElement("label");
        label1.setAttribute("style", "margin-right: 40px;");
        if (item.text && item.text.length > 39)
          item.text = item.text.substr(0,35) + "...";
        label1.setAttribute("value", item.text)
        var hbox3      = document.createElement("hbox");
        var label2     = document.createElement("label");
        if ( "amazon" == item.type )
          label2.setAttribute("style", "font-style: italic; color: #7E001F;");
        else
          label2.setAttribute("style", "font-style: italic;");
        if (item.desc && item.desc.length > 39)
          item.desc = item.desc.substr(0,35) + "...";
        label2.setAttribute("value", item.desc)
        var label3     = document.createElement("label");
        label3.setAttribute("style", "text-decoration: line-through;");
        label3.setAttribute("value", item.strike)
        var hbox2      = document.createElement("hbox");
        hbox2.setAttribute("align", "left");
        var image2     = document.createElement("image"); 
        image2.setAttribute("src", item.star);
        image2.setAttribute("style", "width: 65px; height: 12px;");
        
        vbox1.appendChild(image1);
        hbox.appendChild(vbox1);
        hbox2.appendChild(image2);
        vbox2.appendChild(label1);
        if (item.star)
          vbox2.appendChild(hbox2);
        if (item.strike)
          hbox3.appendChild(label3);
        hbox3.appendChild(label2);
        vbox2.appendChild(hbox3);
        hbox.appendChild(vbox2);
        menuitem.appendChild(hbox);
        if (item.url)
        {
          menuitem.oncommand = function (itemEvent)
          {
            ALX_NS_PH_TB.internalLoadURL(item.url, "newTab");
            if (itemEvent)
              itemEvent.stopPropagation();
          };
          menuitem.addEventListener("command", menuitem.oncommand, false);
        }

        popup.appendChild(menuitem);
      }
      
      if ( !shown && (lastAmazonpopup == id) )
        popup.openPopup(parent, "after_start", 0, 0, true, false);
    },
    
    showpopupsuggestion: function(parent, id, url)
    {
      var value = parent.value;
      value = encodeURIComponent(value);
      loadContentHelp.currentSearchWord = value;
      
      if (trim(value) == "")
      {
        loadContentHelp.showpopuphelp(parent, id, searchboxes["_text"]["searchSuggestionText"]);
        return
      }
      
      loadContentHelp.showpopuphelp(parent, id, searchboxes["_text"]["searchingText"]);
      try {
        url = subst_tokens(url);
        url = url.replace(/__TERM__PLACEHOLDER__/g, value);
      } catch(e) { trace(e) }
      
      var http = new XMLHttpRequest();
      http.open( "GET", url, true );
      http.onload = function()
      {
        if (loadContentHelp.currentSearchWord != value)
          return;
        var popupwindow = document.getElementById(id);
        if (!popupwindow || popupwindow.state != "open")
          return;

        var xml = http.responseXML;
        //trace(http.responseText);
        var items = xml.getElementsByTagName("Item");
        //trace(items.length)
        var itemList = [];
        
        for (var i = 0; i < items.length && itemList.length < 7; ++i)
        {
          
          var item = items[i];
          //trace(item)
          var itemobj = {}
          
          var texts   = item.getElementsByTagName("Text");
          var descs   = item.getElementsByTagName("Description");
          var urls    = item.getElementsByTagName("Url");
          var images  = item.getElementsByTagName("Image");
          
          if (images.length > 0) 
          {
            itemobj["image"] = images[0].getAttribute("source");
            if (itemobj["image"] == "")
              continue;
          } 
          
          if (texts.length > 0) itemobj["text"] = texts[0].textContent; else itemobj["text"]= null;
          if (descs.length > 0) itemobj["desc"] = descs[0].textContent; else itemobj["desc"]= null;
          if (urls.length > 0) itemobj["url"] = urls[0].textContent; else itemobj["url"]= null;
          if (images.length > 0) itemobj["image"] = images[0].getAttribute("source"); else itemobj["image"]= null;
          
          //trace("text: " + itemobj["text"]);
          //trace("desc: " + itemobj["desc"]);
          //trace("url: " + itemobj["url"]);
          //trace("image: " + itemobj["image"]);
          itemList.push(itemobj);
        }

        if (itemList.length == 0 || items.length == 0)
        {
          loadContentHelp.showpopuphelp(parent, id, searchboxes["_text"]["notAvailableText"]);
          return;
        }
        
        loadContentHelp.renderResult(parent, id, itemList);
      };
      http.send(null);
    }
  };
                  
  function trace(msg)
  {
    Cc["@mozilla.org/consoleservice;1"]
    .getService(Ci.nsIConsoleService)
    .logStringMessage( "<" + String( msg ) + ">" );
  }
  function trim(str)
  {
    return str.replace(/^\s+|\s+$/g, "");
  }

  function sq(str)
  {
    return "decodeURIComponent('" + encodeURIComponent(str) + "')";
  }

  function sq_nostr(str)
  {
    return decodeURIComponent(encodeURIComponent(str));
  }

  function GM_hitch(obj, meth) {
    if (!obj[meth]) {
      throw "method '" + meth + "' does not exist on object '" + obj + "'";
    }
    
    var staticArgs = Array.prototype.splice.call(arguments, 2, arguments.length);
  
    return function() { 
      // make a copy of staticArgs (don't modify it because it gets reused for
      // every invocation).
      var args = staticArgs.concat();
  
      // add all the new arguments
      for (var i = 0; i < arguments.length; i++) {
        args.push(arguments[i]);
      }
  
      // invoke the original function with the correct this obj and the combined
      // list of static and dynamic arguments.
      return obj[meth].apply(obj, args);
    };
  }

  function FeedLoader(listener)
  {
    this.listener = listener;
  }
  
  FeedLoader.prototype =
  {
    load: function(url)
    {      
      this.channel = ioService.newChannelFromURI(ioService.newURI(url, null, null));        
      this.channel.asyncOpen(this, null);
    },

    onStartRequest: function(request, context)
    {
      var channel = request.QueryInterface(Ci.nsIChannel);
      this.processor = Cc["@mozilla.org/feed-processor;1"]
                        .createInstance(Ci.nsIFeedProcessor);
      this.processor.listener = this.listener;
      this.processor.parseAsync(null, channel.URI);  
      this.processor.onStartRequest(request, context);
    },
  
    onStopRequest: function(request, context, status)
    {
      if (this.processor)
      {
        this.processor.onStopRequest(request, context, status);
        this.processor = null;
      }
      
      var httpChannel = request.QueryInterface(Ci.nsIHttpChannel);
      var statusCode = httpChannel.responseStatus;
      var headers = {};
      if (statusCode >= 400)  
      {     
        httpChannel.visitResponseHeaders({
          visitHeader: function(name, value)
          {
              headers[name] = value;
          }
        });
        this.listener.handleError(statusCode, headers);
      }
    
    },
  
    onDataAvailable: function(request, context, inputStream,
                              sourceOffset, count)
    {
      if (this.processor)
        this.processor.onDataAvailable(request, context, inputStream,
                                       sourceOffset, count);
    },
 
    QueryInterface: function(iid) {
      if (iid.equals(Ci.nsIStreamListener) ||
          iid.equals(Ci.nsIRequestObserver)||
          iid.equals(Ci.nsISupports))
        return this;
      throw Cr.NS_ERROR_NO_INTERFACE;
    },
  }
  
  function feedWrapper(properties, rFunc)
  {
    this.properties     = null;
    
    if ( properties )
    {
      this.properties = properties;
      this.loadInitMenuFunc();
      this.loadHashItemsFunc();
      this.loadParseItemsFunc();
      this.loadLoadItemFunc();
      this.loadNewItemsFunc();
      this.loadLoadHeadlinesFunc();
      this.loadErrorFunc();
    }
    
    if (rFunc)
      this.refreshFunc = rFunc;
    else
      this.refreshFunc = ALX_NS_PH_TB.rss.fillRSSButton;

    this.prefix = properties.id;
    this.pref = "extensions.alexa.active-buttons." + this.prefix + ".rss";
    this.prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
    this.prefBranch.addObserver(this.pref + ".items", this, false);
    
    if (!this.prefBranch.prefHasUserValue(this.pref + ".hideHeadLine"))
      this.prefBranch.setBoolPref(this.pref + ".hideHeadLine", false); 
  }
  
feedWrapper.prototype = 
  {
    openIn        : "currentTab",    
    prefix        : null,
    buttonId      : null,
    url             : null,
    icons           : null,
    numHeadlines  : null,
    maxHeadlineLength : null,
    maxHeadlineWidth : null,
    minHeadlineWidth : null,
    randomize       : null,
    pref          : null,
    prefBranch    : null,
    
    onErrorFunc   : null,
    initMenuFunc  : null,
    hashItemsFunc : null,
    parseItemsFunc : null,
    loadItemsFunc : null,
    newItemsFunc    : null,
    loadHeadlinesFunc : null,
    errorFunc : null,
    itemList        : [],
    
    loadProperties: function(properties)
    {
      if (properties.openIn)
      {
        this.openIn       = properties.openIn;
      }
      if (properties.refreshText)
        this.refreshText      = properties.refreshText;
      else
        this.refreshText      = "Refresh";

      this.buttonId       = "ALX_NS_PH_TB-RSS-" + this.prefix + "-Menu"
      this.url            = properties.feedUrl;
      this.urls           = properties.feedUrls;
      this.icons          = properties.icons;
      this.numHeadlines   = properties.numHeadlines;
      this.maxHeadlineLength = properties.maxHeadlineLength;
      this.minHeadlineWidth = properties.minHeadlineWidth;
      this.maxHeadlineWidth = properties.maxHeadlineWidth;
      this.randomize      = properties.randomize;      
      this.nothingAvailable = properties.nothingAvailable;
    },

    observe: function(subject, topic, data)
    {
      if (topic == "nsPref:changed")
      {
        var items = this.prefBranch.getCharPref(this.pref + ".items");
        this.itemList = nativeJSON.decode(items)
        this.load(fresh);
      }
    },

    load: function(fresh)
    {
      this.loadProperties(this.properties);
      if (fresh)
      {
        this.parseItemsFunc(this);
        this.prefBranch.setCharPref(this.pref + ".items", encodeURIComponent(nativeJSON.encode(this.itemList)));
      }
      this.loadItemsFunc(this);
      this.loadHeadlines(this);
    },

    toggleHeadlines: function(fwObj)
    {
      var hidden = this.prefBranch.getBoolPref(this.pref + ".hideHeadLine");
      var hswitch = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button-switch");
      for (var i = 0; i < fwObj.numHeadlines; i++)
      {
        var rssButton = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + i);
        
        if (hidden)
          rssButton.style.display = "block";
        else
          rssButton.style.display = "none";
        
        if ( i != 0 )
        {
          var intersec = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + i + "-interpunct");
          if (hidden)
            intersec.style.display = "block";
          else
            intersec.style.display = "none";
        }
      }
      
      this.prefBranch.setBoolPref(this.pref + ".hideHeadLine", !hidden);
      
      if (hidden)
        hswitch.setAttribute("label", "Hide Headlines");
      else
        hswitch.setAttribute("label", "Show Headlines");
      
    },

    shutdown: function()
    {
      this.prefBranch.removeObserver(this.pref + ".items", this);
    },

    loadInitMenuFunc : function(func)
    {
      if ( func )
      {
        this.initMenuFunc = func;
      }
       else
        this.initMenuFunc = function(fwObj)
        {
          var menupopup = document.getElementById(fwObj.buttonId);          
          while (menupopup.firstChild) {
            menupopup.removeChild(menupopup.firstChild);
          }
          
          if (this.prefix != "amazontweet")
          {
            var menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", this.refreshText);
            menuitem.oncommand = function(event)
            {
              ALX_NS_PH_TB.log(fwObj.prefix + ':i');
              fwObj.refreshFunc(fwObj.properties, true);
              event.stopPropagation();
            };
            menuitem.addEventListener("command", menuitem.oncommand, false);
            menupopup.appendChild(menuitem);
          }
        };
    },
    
    loadHashItemsFunc: function(func)
    {
      if ( func )
        this.hashItemsFunc = func;
      else
        this.hashItemsFunc = function(fwObj)
        {
          var converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Ci.nsIScriptableUnicodeConverter);
          converter.charset = "UTF-8";
          var md5 = Cc["@mozilla.org/security/hash;1"].createInstance(Ci.nsICryptoHash);
          md5.init(md5.MD5);
          for (var i = 0; i < fwObj.itemList.length; i++)
          {
            var item = fwObj.itemList[i];
            var data = null;
            var key = null;
            var result = {};
            
            if (item.title)
              key = item.title
            else if (item.text)
              key = item.text
            data = converter.convertToByteArray(key, result);
            md5.update(data, data.length);
            
            if (item.link)
              key = item.link
            else if (item.url)
              key = item.url
            data = converter.convertToByteArray(key, result);
            md5.update(data, data.length);
            
            if (item.image)
            {
              key = item.image;
              data = converter.convertToByteArray(key, result);
              md5.update(data, data.length);
            }
            
            if (item.desc)
            {
              key = item.desc;
              data = converter.convertToByteArray(key, result);
              md5.update(data, data.length);
            }
          }
          var new_hash = md5.finish(true);
          var hash = "";
          try
          {
            hash = fwObj.prefBranch.getCharPref(fwObj.pref + ".hash");
          }
          catch (ex)
          {
          }
          if (hash && new_hash != hash)
          {
            fwObj.newItemsFunc(fwObj);
          }
          if (new_hash != hash)
          {
            fwObj.prefBranch.setCharPref(fwObj.pref  + ".hash", new_hash);
          }
          
        };  
    },
    
    loadParseItemsFunc: function(func)
    {
      if ( func )
        this.parseItemsFunc = func;
      else
        this.parseItemsFunc = function(fwObj)
        {
          ;
        };  
    },
    
    loadErrorFunc : function(func)
    {
      if ( func )
        this.errorFunc = func;
      else
        this.errorFunc = function(statusCode, headers)
        {
          ;
        };
    },
    
    loadLoadItemFunc: function(func)
    {
      if ( func )
        this.loadItemsFunc = func;
      else
        this.loadItemsFunc = function(fwObj)
        {
          this.initMenuFunc(this);
          var menupopup = document.getElementById(fwObj.buttonId);

          /*menuitem = document.createElement("menuitem");
          menuitem.setAttribute("id", "ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button-switch" );
          if (!this.prefBranch.getBoolPref(this.pref + ".hideHeadLine")) 
            menuitem.setAttribute("label", "Hide Headlines");
          else
            menuitem.setAttribute("label", "Show Headlines");
          menuitem.oncommand = function(event)
          {
            ALX_NS_PH_TB.log(fwObj.prefix + ':t');
            fwObj.toggleHeadlines(fwObj);
            event.stopPropagation();
          };
          menuitem.addEventListener("command", menuitem.oncommand, false);
          menupopup.appendChild(menuitem);*/ 

          if (this.prefix != "amazontweet")
            menupopup.appendChild(document.createElement("menuseparator"));
          if (fwObj.itemList.length > 0)
          {
            for (var i = 0; i < fwObj.itemList.length && i < 20; i++)
            {
              var item = fwObj.itemList[i];
              var title = item.title;
              let link = ioService.newURI(item.link, null, null);
              var menuitem = document.createElement("menuitem");
              menuitem.setAttribute("label", title);
              if (fwObj.icons && fwObj.icons == "true")
              {
                menuitem.setAttribute("class","menuitem-iconic alexa-favicon");
                menuitem.setAttribute("image","http://"+link.host+"/favicon.ico");
              }
              menuitem.setAttribute("tooltiptext", title + "\n" + link.spec)
              menuitem.oncommand = function(event)
              {
                ALX_NS_PH_TB.log(fwObj.prefix + ':i');
                ALX_NS_PH_TB.internalLoadURL(link.spec, fwObj.openIn);
                event.stopPropagation();
              };

              menuitem.addEventListener("command", menuitem.oncommand, false);
              menupopup.appendChild(menuitem);
            }
          } else
          {
            var menuitem = document.createElement("menuitem");
            if (fwObj.nothingAvailable)
              menuitem.setAttribute("label", fwObj.nothingAvailable);
            else 
              menuitem.setAttribute("label", "No data available");
            menuitem.style.color = "gray";
            menuitem.oncommand = function(event)
            {
              event.stopPropagation();
            };
            menuitem.addEventListener("command", menuitem.oncommand, false);
            menupopup.appendChild(menuitem);
          }
        }
    },
    
    loadNewItemsFunc: function(func)
    {
      if ( func )
        this.newItemsFunc = func;
      else
        this.newItemsFunc = function(fwObj)
        {
          fwObj.properties.updated = true;
          ALX_NS_PH_TB.updateButtons(get_data());
        };  
    },
    
    loadLoadHeadlinesFunc: function(func)
    {
      if ( func )
        this.loadHeadlines = func;
      else 
        this.loadHeadlines = function(fwObj)
        {
          if (fwObj.randomize && fwObj.randomize == "true")
          {
            items = shuffle(fwObj.itemList);
          }
          var hist = Cc["@mozilla.org/browser/global-history;2"].getService(Ci.nsIGlobalHistory2);
          var n = 0;
          for (var i = 0; i < 2*(items.length) && n < fwObj.numHeadlines; i++)
          {
            var item = items[i%items.length];
            var title = item.title;
            let link = ioService.newURI(item.link, null, null);
            if (i < items.length && hist.isVisited(link))
            {
              continue;
            }
            var rssButton = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + n);
            rssButton.setAttribute("tooltiptext", title + "\n" + link.spec)
            if (title.length > fwObj.maxHeadlineLength)
            {
              title = title.substr(0,fwObj.maxHeadlineLength - 3) + "\u2026";
            }
            rssButton.setAttribute("label", title);
            rssButton.setAttribute("minwidth", "0");
            if (fwObj.minHeadlineWidth)
            {
              rssButton.setAttribute("flex", "0");              
              rssButton.setAttribute("minwidth", Math.min(fwObj.minHeadlineWidth, rssButton.scrollWidth));
            }
            rssButton.setAttribute("flex", "1");
            if (fwObj.maxHeadlineWidth)
            {
              rssButton.setAttribute("maxwidth", fwObj.maxHeadlineWidth);
            }
            rssButton.setAttribute("oncommand",
              "ALX_NS_PH_TB.log(" + sq(fwObj.prefix) + " + ':h'); " +
              "ALX_NS_PH_TB.internalLoadURL(" + sq(link.spec) + ", " + sq(fwObj.openIn) + ");"
            );
            rssButton.style.display = "block";
            n++;
            var interpunct = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + n + "-interpunct");
            if (interpunct) interpunct.style.display = "block";
          }
          for (; n < fwObj.numHeadlines; n++)
          {
            var rssButton = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + n);
            rssButton.style.display = "none";
            var interpunct = document.getElementById("ALX_NS_PH_TB-RSS-" + fwObj.prefix + "-Button" + n + "-interpunct");
            if (interpunct) interpunct.style.display = "none";
          }
          ALX_NS_PH_TB.updatePopups();
        }
    },
        
    loadRss: function(nocache)
    {
      var fwObj          = this;
      
      if(this.properties)
      {
        this.loadProperties(this.properties);
        var now = new Date();
        try
        {
          var last = parseInt(fwObj.prefBranch.getCharPref(fwObj.pref + ".time"), 10);
        }
        catch (ex)
        {
        }

        if (!nocache)
        {
          try
          {
            if (last + this.properties.refreshInterval * 1000 > now.getTime())
            {
              var items = this.prefBranch.getCharPref(this.pref + ".items");
              this.itemList = nativeJSON.decode(decodeURIComponent(items));
              fwObj.load(false);
              return;
            }
          }
          catch (ex)
          {
          }
          fwObj.prefBranch.setCharPref(fwObj.pref + ".time", now.getTime());
        }

        var visualized = (this.properties.visualized == "true");

        var urls = this.urls;
        if ( !urls || urls.length == 0)
          urls = [this.url];
        
        var feedDownloaded = [];
        for(var i = 0; i < urls.length; ++i)
        {
          feedDownloaded.push(null);
        }
        fwObj.itemList = [];

        for (var i = 0; i < urls.length; ++i)
        {
          let seq = i;
          let url = urls[i];
          if (visualized)
          {
            var popupid = fwObj.buttonId;
            var menuitem = popupid.parentNode;
            var feedLoader = new FeedLoader({
              handleResult: function(result) {
                var feed = result.doc.QueryInterface(Ci.nsIFeed);

                if (!feed)
                  return;
                
                feedDownloaded[seq] = [];
                for (var i = 0; i < feed.items.length && i < 7; i++)
                {
                  var item = feed.items.queryElementAt(i, Ci.nsIFeedEntry);
                  var title = item.title.plainText();
                  var link = item.link;
                  var pubDate = Date.parse( item.updated );

                  loadContentHelp.HTMLParser(item.summary.text)
                  var imgsrc = null;
                  var desc   = null;
                  try {
                    var divdoc = loadContentHelp.HTMLParser(item.summary.text);
                    imgsrc = divdoc.getElementsByTagName("img")[0].src;
                    desc = divdoc.getElementsByTagName("font")[1].textContent;
                  }catch(e) {trace(e)}
                  feedDownloaded[seq].push({"text" : title, "url": link.spec, "image": imgsrc, "desc": desc, "pubDate": pubDate});
                }
        
                for (var j = 0;  j < feedDownloaded.length; ++j)
                {
                  if (feedDownloaded[j] == null)
                  {
                    return;
                  }
                }

                while ( fwObj.itemList.length <= 20 )
                {
                  var node  = null;
                  var maxIndex = null;
                  
                  for (var j = 0;  j < feedDownloaded.length; ++j)
                  {
                    if (feedDownloaded[j].length > 0)
                    {
                      if (node == null)
                      {
                        node = feedDownloaded[j][0];
                        maxIndex = j;
                      } else
                      {
                        if (feedDownloaded[j][0].pubDate > node.pubDate)
                        {
                          node = feedDownloaded[j][0];
                          maxIndex = j;
                        }
                      }
                    }
                  }
                  
                  if (node  == null)
                    break;
                  
                  node = feedDownloaded[maxIndex].shift();
                  fwObj.itemList.push(node);
                }
                
                fwObj.hashItemsFunc(fwObj);
                loadContentHelp.renderResult(menuitem, popupid, fwObj.itemList, true);
              },
              handleError: this.errorFunc
            });
          }
          else
          {
            var feedLoader = new FeedLoader({
              handleResult: function(result) {
                var feed = result.doc.QueryInterface(Ci.nsIFeed);
              
                if (!feed)
                  return;
                // read the item and hash it 
                feedDownloaded[seq] = [];

                for (var j = 0; j < feed.items.length; j++)
                {
                  var item = feed.items.queryElementAt(j, Ci.nsIFeedEntry);
                  var title = item.title.plainText();
                  var link = item.link;
                  var pubDate = Date.parse( item.updated );

                  feedDownloaded[seq].push({"title" : title, "link": link.spec, "pubDate": pubDate});
                }

                for (var j = 0;  j < feedDownloaded.length; ++j)
                {
                  if (feedDownloaded[j] == null)
                  {
                    return;
                  }
                }

                while ( fwObj.itemList.length <= 20 )
                {
                  var node  = null;
                  var maxIndex = null;
                  
                  for (var j = 0;  j < feedDownloaded.length; ++j)
                  {
                    if (feedDownloaded[j].length > 0)
                    {
                      if (node == null)
                      {
                        node = feedDownloaded[j][0];
                        maxIndex = j;
                      } else
                      {
                        if (feedDownloaded[j][0].pubDate > node.pubDate)
                        {
                          node = feedDownloaded[j][0];
                          maxIndex = j;
                        }
                      }
                    }
                  }
                  
                  if (node  == null)
                    break;
                  
                  node = feedDownloaded[maxIndex].shift();
                  fwObj.itemList.push(node);
                }
                
                fwObj.hashItemsFunc(fwObj);
                fwObj.load(true);
              },
              handleError: function () {
                feedDownloaded[seq] = [];
                for (var j = 0;  j < feedDownloaded.length; ++j)
                {
                  if (feedDownloaded[j] == null)
                  {
                    return;
                  }
                }
                fwObj.loadProperties(fwObj.properties);
                fwObj.loadItemsFunc(fwObj);
              } 
            });
          }
          feedLoader.load(url);
        }
      }
    }
  }

  var sandboxedRequest = function (myURLString, callbackFunc, method, databundle)
  {
    // global channel
    var gChannel;
    var statusCode;
    // init the channel
    
    // the IO service
    var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
    
    // create an nsIURI
    var uri = ioService.newURI(myURLString, null, null);
    
    // get a channel for that nsIURI
    gChannel = ioService.newChannelFromURI(uri);
    
    var cookieListener = {
      observe : function(aSubject, aTopic, aData) {
        //trace(aSubject);
        //trace(aTopic);
        // Make sure it is our connection first.
        if (aSubject == gChannel) {
          var httpChannel = aSubject.QueryInterface(Ci.nsIHttpChannel);
          if (aTopic == "http-on-modify-request") {
             httpChannel.setRequestHeader("Cookie", "", false);
          } else if (aTopic == "http-on-examine-response") {
             statusCode = httpChannel.responseStatus;
             httpChannel.setResponseHeader("Set-Cookie", "", false);
          }
        }
      },
    
      QueryInterface : function(aIID) {
        if (aIID.equals(Ci.nsISupports) ||
            aIID.equals(Ci.nsIObserver))
          return this;
        throw Cr.NS_NOINTERFACE;
      }
    };
    
    function StreamListener(aCallbackFunc) {
      //trace("Tracing on start request");
      this.mCallbackFunc = aCallbackFunc;
    }
    
    StreamListener.prototype = {
      mData: "",
      sandboxlistener : true,
    
      // nsIStreamListener
      onStartRequest: function (aRequest, aContext) {
        this.mData = "";
      },
    
      onDataAvailable: function (aRequest, aContext, aStream, aSourceOffset, aLength) {
        var scriptableInputStream = 
          Cc["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Ci.nsIScriptableInputStream);
        scriptableInputStream.init(aStream);
    
        this.mData += scriptableInputStream.read(aLength);
      },
    
      onStopRequest: function (aRequest, aContext, aStatus) {
        if (Components.isSuccessCode(aStatus)) {
          // request was successfull
          this.mCallbackFunc(this.mData, statusCode);
        } else {
          // request failed
          this.mCallbackFunc(this.mData, statusCode);
        }
        gChannel = null;
        observerService.removeObserver(cookieListener, "http-on-modify-request");
        observerService.removeObserver(cookieListener, "http-on-examine-response");
      },
    
      // nsIChannelEventSink
      onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
        // if redirecting, store the new channel
        gChannel = aNewChannel;
      },
      
      // nsIInterfaceRequestor
      getInterface: function (aIID) {
        try {
          return this.QueryInterface(aIID);
        } catch (e) {
          throw Cr.NS_NOINTERFACE;
        }
      },
    
      // nsIProgressEventSink (not implementing will cause annoying exceptions)
      onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },
      onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },
    
      // nsIHttpEventSink (not implementing will cause annoying exceptions)
      onRedirect : function (aOldChannel, aNewChannel) { },
    
      // we are faking an XPCOM interface, so we need to implement QI
      QueryInterface : function(aIID) {
        if (aIID.equals(Ci.nsISupports) ||
            aIID.equals(Ci.nsIInterfaceRequestor) ||
            aIID.equals(Ci.nsIChannelEventSink) || 
            aIID.equals(Ci.nsIProgressEventSink) ||
            aIID.equals(Ci.nsIHttpEventSink) ||
            aIID.equals(Ci.nsIStreamListener))
          return this;
    
        throw Cr.NS_NOINTERFACE;
      }
    };
    
    // get the observer service and register for the two coookie topics.
    var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
    observerService.addObserver(cookieListener, "http-on-modify-request", false);
    observerService.addObserver(cookieListener, "http-on-examine-response", false);
    
    
    // get an listener
    var listener = new StreamListener(callbackFunc);
    
    /*for (var i in listener)
    {
      trace("listener: " + i);
    }*/
    
    if (method && method == "POST")
    {
      var inputStream = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(Ci.nsIStringInputStream);
      var postData = "";
      var type = "application/x-www-form-urlencoded";
      if(databundle)
      {
        postData = databundle.data;
        type = databundle.type;
      }
      inputStream.setData(postData, postData.length);
      var uploadChannel = gChannel.QueryInterface(Ci.nsIUploadChannel);
      uploadChannel.setUploadStream(inputStream, type, -1);
      
      var httpChannel = gChannel.QueryInterface(Ci.nsIHttpChannel);
      httpChannel.requestMethod = "POST";
    }
    gChannel.notificationCallbacks = listener;
    gChannel.asyncOpen(listener, null);
  }

  function GM_xmlhttpRequester() {
  }
  
  // this function gets called by user scripts in content security scope to
  // start a cross-domain xmlhttp request.
  //
  // details should look like: 
  // {method,url,onload,onerror,onreadystatechange,headers,data}
  // headers should be in the form {name:value,name:value,etc}
  // can't support mimetype because i think it's only used for forcing
  // text/xml and we can't support that
  GM_xmlhttpRequester.prototype.contentStartRequest = function(details) {
    // don't actually need the timer functionality, but this pops it 
    // out into chromeWindow's thread so that we get that security 
    // context.
  
    // important to store this locally so that content cannot trick us up with
    // a fancy getter that checks the number of times it has been accessed, 
    // returning a dangerous URL the time that we actually use it.
    var url = details.url;
    
    // make sure that we have an actual string so that we can't be fooled with
    // tricky toString() implementations.
    if (typeof url != "string") {
      throw new Error("Invalid url: url must be of type string");
    }
    
    var scheme = ioService.extractScheme(url);
  
    // This is important - without it, GM_xmlhttpRequest can be used to get
    // access to things like files and chrome. Careful.
    switch (scheme) {
      case "http":
      case "https":
      case "ftp":
        this.chromeStartRequest(url, details);
        break;
      default:
        throw new Error("Invalid url: " + url);
    }
  
  }

  // this function is intended to be called in chrome's security context, so 
  // that it can access other domains without security warning
  GM_xmlhttpRequester.prototype.chromeStartRequest = function(safeUrl, details) {
    var req = new XMLHttpRequest();
    if (details.mozBackgroundRequest)
    {
      req.mozBackgroundRequest = true;
    }
  
    this.setupRequestEvent(req, "onload", details);
    this.setupRequestEvent(req, "onerror", details);
    this.setupRequestEvent(req, "onreadystatechange", details);
  
    req.open(details.method, safeUrl);
  
    if (details.headers) {
      for (var prop in details.headers) {
        req.setRequestHeader(prop, details.headers[prop]);
      }
    }
  
    req.send(details.data);
  }
  
  // arranges for the specified 'event' on xmlhttprequest 'req' to call the
  // method by the same name which is a property of 'details' in the content
  // window's security context.
  GM_xmlhttpRequester.prototype.setupRequestEvent = 
  function(req, event, details) {
    if (details[event]) {
      req[event] = function() {
        var responseState = {
          // can't support responseXML because security won't
          // let the browser call properties on it
          responseText:req.responseText,
          readyState:req.readyState,
          responseHeaders:(req.readyState == 4 ? 
                           req.getAllResponseHeaders() : 
                           ''),
          status:(req.readyState == 4 ? req.status : 0),
          statusText:(req.readyState == 4 ? req.statusText : '')
        }
  
        // Call event handler
        var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        timer.initWithCallback({
          notify: function(){details[event](responseState);}
        }, 0, Ci.nsITimer.TYPE_ONE_SHOT);
        timers.push(timer);
      }
    }
  
  }



  function getElementValue(button, tag)
  {
    var tags = button.getElementsByTagName(tag);
    if (tags && tags.length)
      return tags.item(0).nodeValue;
    return "";
  }
  var toolbar_buttons = {};
  var toolbar_global  = {};
  var toolbar_headlines = [];
  var timers = [];
  var events = [];
  var pageturn_listeners = [];

  function add_button(toolbar, button, filename)
  {
    var toolbaritem = document.createElement("toolbaritem");
    toolbaritem.setAttribute("flex", "0");
    toolbaritem.setAttribute("align", "center");
    var properties = {};
    var menuItemNodes = null;
    var menuItems = [];

    // XML -> javascript
    {
      var id = button.getAttribute("id");
      if (id)
        properties["id"] = id;

      var children = button.childNodes;
      for (var i = 0; i < children.length; i++)
      {
        var item = children.item(i);
        var tagName = item.tagName;
        switch (tagName)
        {
          case "type":
          case "text":
          case "image":
          case "hoverImage":
          case "updatedImage":
          case "updatedHoverImage":
          case "imageType":
          case "url": 
          case "urlRegexFrom":
          case "urlRegexTo":
          case "label": 
          case "width":
          case "openIn":
          case "tooltip":
          case "id":
          case "menuUrl":
          case "menuImage":
          case "menuIcon":
          case "noPriceAvailable":
          case "nothingAvailable":
          case "moreRelatedLinksText":
          case "disableText":
          case "notAvailableText":
          case "loadingErrorText":
          case "searchingText":
          case "searchSuggestionText":
          case "refreshText":
          case "suggestionUrl":
          case "runat":
          case "trigger":
          case "contentUrl":
          case "orientation":
          case "defaultText":
          case "feedUrl": 
          case "feedUrlUrl": 
          case "numHeadlines":
          case "minHeadlineWidth":
          case "maxHeadlineWidth":
          case "maxHeadlineLength":
          case "windowFeatures":
          case "refreshInterval":
          case "randomize":
          case "icons":
          case "javascriptUrl":
          case "javascript":
          case "shortText":
          case "visualized":
            var localeId  = item.getAttribute("id");
            properties[tagName] = loadContentHelp.getLocalName(item.textContent, id, localeId);
            if (tagName == "feedUrl" ) 
            {
              if (properties["feedUrls"])
                properties["feedUrls"].push(properties[tagName]);
              else
                properties["feedUrls"] = [ properties[tagName] ];
            }
            break;
          case "menuItems":
            menuItemNodes = item.childNodes;
            break;
        }
      }
      if (properties["type"] == "catalog" || properties["type"] == "multiRss")
        properties["type"] = "logo";
      if (properties["type"] == "amazonRss")
        properties["type"] = "rss";
      // Grab configuration for menus (e.g. searchBuddy menu, logo menu)
      menuItems = loadContentHelp.loadXmlMenuToProperty(menuItemNodes);
    }

    if (properties.id)
    {
      toolbaritem.setAttribute("id", "ALX_NS_PH_TB-Item-" + properties.id);
      if (loadContentHelp.stringBundle && loadContentHelp.stringBundle[id] && !loadContentHelp.stringBundle[id]["visible"])
        return;
    }

    var toolbarchild = null;
    switch (properties.type)
    {
      case "search":
        toolbarchild = document.createElement("textbox");
        break;
      case "logo":
      case "simpleRss":
      case "searchBuddy":
      case "url":
      case "javascript":
      case "siteRank":
      case "relatedLinks":
      case "rss":
        toolbarchild = document.createElement("toolbarbutton");
        //toolbarchild.style.MozBinding = "url(chrome://ALX_NS_PH/content/button.xml#alexa-menu-button)";
        toolbarchild.style.cursor = "pointer";
        toolbarchild.setAttribute("validate", "never");
        toolbarchild.orient = "horizontal";
        break;
      default:
        return;
    }

    toolbarchild.setAttribute("id", "ALX_NS_PH_TB-Button-" + properties.id);
    toolbaritem.appendChild(toolbarchild);
    
    if (properties.orientation && properties.orientation == "right")
    {
      toolbar.appendChild(toolbaritem);
    }
    else
    {
      var toolbarspring = document.getElementById("ALX_NS_PH_TB-Toolbar-Spring");
      toolbar.insertBefore(toolbaritem, toolbarspring);
    }

    var id = properties.id;
    if (id)
    {
      ALX_NS_PH_TB.log(id + ":l");
    }

    if (properties.label)
      toolbarchild.setAttribute("label", properties.label);

    var openIn = "currentTab";
    if (properties.openIn)
      openIn = properties.openIn;

    var windowFeatures = "";
    if (properties.windowFeatures)
      windowFeatures = properties.windowFeatures;

    if (properties.searchingText)
      searchboxes["_text"]["searchingText"] = properties.searchingText;
    if (properties.searchSuggestionText)
      searchboxes["_text"]["searchSuggestionText"] = properties.searchSuggestionText;
    if (properties.notAvailableText)
      searchboxes["_text"]["notAvailableText"] = properties.notAvailableText;
    if (properties.loadingErrorText)
      searchboxes["_text"]["loadingErrorText"] = properties.loadingErrorText;

    if (properties.type == "simpleRss")
    {
      if (properties.feedUrl)
      {
        toolbarchild.setAttribute("type", "menu-button");
        var menupopup1 = document.createElement("menupopup");
        let popupid= "ALX_NS_PH_TB-Button-" + properties.id + "-popup";

        menupopup1.setAttribute("id", popupid);
        loadContentHelp.requestAndAppendPopup(properties, toolbarchild, menupopup1, properties.feedUrl, false, null);
        menupopup1.addEventListener("popupshowing", menupopup1.onpopupshowing, false);
        toolbarchild.appendChild(menupopup1);
      } else
        toolbarchild.setAttribute("type", "button");
    }

    if (properties.type == "logo")
    {
      // Global variable "logo" is used to specify
      // which item the popup bubble comes from on startup
      if (logo == null)
        logo = toolbaritem;

      if (menuItems && menuItems.length)
      {
        toolbarchild.setAttribute("type", "menu-button");
        loadContentHelp.loadMenu(toolbarchild, properties, menuItems);
      } else if (properties.menuUrl){
        toolbarchild.setAttribute("type", 'menu-button');
        loadContentHelp.loadMenuFromUrl(toolbarchild, properties);
      }
    }
    else if (properties.type == "relatedLinks")
    {
      var moreRelatedLinksText = "More Related Links";
      var disableText = "Disabled";
      var notAvailableText = "Not Available";
      var loadingText = "loading";
      
      if (properties.moreRelatedLinksText)
        moreRelatedLinksText = properties.moreRelatedLinksText;
      if (properties.disableText)
        disableText = properties.disableText;
      if (properties.notAvailableText)
        notAvailableText = properties.notAvailableText;
      if (properties.loadingText)
        loadingText = properties.loadingText;

      toolbarchild.setAttribute("type", "menu-button");
      var menupopup = document.createElement("menupopup");
      menupopup.setAttribute("onpopupshowing", "ALX_NS_PH_TB.log(" + sq(id) + " + ':m'); ALX_NS_PH_SPARKLINE.updateRelatedLinks(" + sq(id) + ", this, '" + moreRelatedLinksText +  "', '"  + disableText +  "' , '" + notAvailableText + "' , '" + loadingText +"' );");
      toolbarchild.appendChild(menupopup);
    }
    else if (properties.type == "searchBuddy")
    {
      var iid     = searchboxes.previousSearchbox;
      searchboxes.previousSearchbox = null;

      var func = function() 
      {
        if (menuItems && menuItems.length)
        {
          if (menuItems.length > 1)
            toolbarchild.setAttribute("type", "menu-button");
          var menupopup = document.createElement("menupopup");
          menupopup.setAttribute("onpopupshowing", "ALX_NS_PH_TB.log(" + sq(id) + " + ':m');");
          toolbarchild.appendChild(menupopup);

          for (var i = 0; i < menuItems.length; i++)    
          {
            var menuItemProperties = menuItems[i];
            let url = menuItemProperties.url;
            let displayName = menuItemProperties.displayName;
            let defaultText = menuItemProperties.defaultText;
            let suggestionUrl = menuItemProperties.suggestionUrl;

            if (menuItemProperties.type == "provider")
            {
              var menuitem = document.createElement("menuitem");
              menuitem.setAttribute("tooltiptext", displayName);
              menuitem.setAttribute("label", displayName);
              searchboxes[iid].showWindow = true;

              if (menuItemProperties.icon && menuItemProperties.icon == "true" && menuItemProperties.image)
              {
                menuitem.setAttribute("class","menuitem-iconic alexa-favicon");
                menuitem.setAttribute("image", ALX_NS_PH_TB.checkLocalDefault(menuItemProperties.image));
              }
              menuitem.oncommand = function(event)
              {
                ALX_NS_PH_TB.log(id + ':i');
                
                searchboxes[iid]["index"] = -1; 
                if (defaultText)
                  searchboxes[iid].defaultText = defaultText;
                else
                  searchboxes[iid].defaultText = null;

                toolbarchild.setAttribute("tooltiptext", displayName);
                properties.tooltip = displayName;

                var children = menupopup.childNodes
                for (var k = 0; k < menupopup.childNodes.length; ++k)
                {
                  menupopup.childNodes[k].style.fontWeight = "";
                  menupopup.childNodes[k].style.fontStyle = "";
                }
                this.setAttribute("style", "font-weight: bold; font-style: italic");
                searchboxes[iid].url = url;
                
                var searchbody = document.getElementById(iid);
                
                if (suggestionUrl)
                  searchboxes[iid]["suggestionUrl"] = suggestionUrl;
                else
                  searchboxes[iid]["suggestionUrl"] = null;

                if (defaultText)
                {
                  var focused = searchbody.getAttribute("focused");
                  if ( searchboxes[iid]["isDefaultText"] == true)
                    searchbody.value = defaultText;
                  else if ( searchbody.value.replace(/\s/gi,"") == "" && !focused)
                  {
                    searchbody.value = defaultText;
                    loadContentHelp.onDefaultText(searchbody);
                    searchboxes[iid]["isDefaultText"] = true;
                  }
                } else
                {
                  if ( searchboxes[iid]["isDefaultText"] == true)
                  {
                    searchbody.value = "";
                    loadContentHelp.offDefaultText(searchbody);
                    searchboxes[iid]["isDefaultText"] = false;
                  }
                }

                if (searchboxes[iid]["suggestionUrl"])
                {
                  //trace("in");
                  if (searchboxes[iid]["isDefaultText"] == false)
                  {
                    if (trim(searchbody.value) == "")
                    {
                      var tmp = function () {loadContentHelp.showpopuphelp(searchbody, iid + '-popup', searchboxes["_text"]["searchSuggestionText"])};
                      window.setTimeout(tmp, 10);
                    }
                    else
                    {
                      var tmp = function () {loadContentHelp.showpopupsuggestion(searchbody, iid + '-popup', searchboxes[iid]["suggestionUrl"])};
                      window.setTimeout(tmp, 10);
                    }
                  }
                }
                event.stopPropagation();
              };
              menuitem.addEventListener("command", menuitem.oncommand, false);

              if (i == 0)
              {
                if (url)
                  searchboxes[iid].url = url;
                if (defaultText)
                {
                  searchboxes[iid]["defaultText"] = defaultText;
                  searchboxes[iid]["isDefaultText"] = true;
                  var searchbody = document.getElementById(iid);
                  if (searchboxes[iid]["isDefaultText"] == true)
                  {
                    searchbody.value = defaultText;
                    loadContentHelp.onDefaultText(searchbody);
                  } else if ( searchbody.value.replace(/\s/gi,"") == "" )
                  {
                    searchbody.value = defaultText;
                    loadContentHelp.onDefaultText(searchbody);
                  }
                }
                menuitem.setAttribute("style", "font-weight: bold; font-style: italic");
                toolbarchild.setAttribute("tooltiptext", displayName);
                properties._oncommand = function(event)
                {
                  ALX_NS_PH_TB.log(id + ':c');
                  
                  searchboxes[iid].showWindow = false;
                  var func = function () { searchboxes[iid].showWindow = true; };
                  window.setTimeout(func, loadContentHelp.searchTimeout);
                        
                  var popup = document.getElementById(id + '-popup');
                  if (popup)
                    popup.hidePopup();

                  ALX_NS_PH_TB.loadSearchURL(event, searchboxes[iid]["url"], searchboxes[iid]["openId"], iid);
                };
              }
              menupopup.appendChild(menuitem);
            }
            else if (menuItemProperties.type == "menuBreak")
            {
              var menuseparator = document.createElement("menuseparator");
              menupopup.appendChild(menuseparator);
            }
          }
        } else {
          properties._oncommand = function(event)
          {
            ALX_NS_PH_TB.log(id + ':c');

            searchboxes[iid].showWindow = false;
            var func = function () { searchboxes[iid].showWindow = true; };
            window.setTimeout(func, loadContentHelp.searchTimeout);
                  
            var popup = document.getElementById(id + '-popup');
            if (popup)
              popup.hidePopup();

            ALX_NS_PH_TB.loadSearchURL(event, searchboxes[iid]["url"], searchboxes[iid]["openId"], iid);
          };
        }
      }

      if (!properties.menuUrl)
        func();
      else
      {
        properties.menuUrl = ALX_NS_PH_TB.checkLocalDefault(properties.menuUrl);
        var buddyhttp = new XMLHttpRequest();
        buddyhttp.open( "GET", properties.menuUrl, true );
        buddyhttp.onload = function()
        {
          var xml = buddyhttp.responseXML;
          if (xml.firstChild.tagName == "menuItems")
            menuItemNodes = xml.firstChild.childNodes;
          else
            menuItemNodes = xml.childNodes;
            
          menuItems = loadContentHelp.loadXmlMenuToProperty(menuItemNodes);
          func();
          ALX_NS_PH_TB.updateButtons(get_data());
        }
        buddyhttp.send(null);
      }
    }
    else if (properties.type == "rss")
    {
      // create dropdown menu and set up id and log listener
      toolbarchild.setAttribute("type", "menu-button");      
      var menupopup = document.createElement("menupopup");
      // get the fillin function

      var rssCallback = function ()
      {
        var fillRSSButton = ALX_NS_PH_TB.rss.fillRSSButton;
        ALX_NS_PH_TB.rss.initRSSButton(toolbaritem, toolbarchild, properties, menupopup, fillRSSButton);
        var refreshInterval = properties.refreshInterval * 1000;      
      
      
        if (refreshInterval > 0)
        {
          var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
          timer.initWithCallback({
            notify: function()
            {
              fillRSSButton(properties, false);
            }
          }, refreshInterval, Ci.nsITimer.TYPE_REPEATING_SLACK);
          timers.push(timer);
        }
      
        // fill the button
        fillRSSButton(properties, false);
      }

      if (properties.feedUrlUrl)
        loadContentHelp.loadFeedsFromUrl(properties, rssCallback);
      else
        rssCallback();
    }
    else if (properties.type == "search")
    {
      var id = toolbarchild.getAttribute("id");
      var url = properties.url;
      if (properties.width)
        toolbarchild.setAttribute("width", properties.width);

      toolbarchild.setAttribute("noautofocus", "true");
      searchboxes[id] = {url: properties.url, openIn: openIn, suggestionUrl: null, isDefaultText: false, index: -1};
      var popupset = document.getElementById("ALX_NS_PH_TB-Toolbar-popupset");
      var popup = document.createElement("popup");
      popup.setAttribute("id", id + "-popup");
      popup.setAttribute("minwidth", "150px");
      popup.setAttribute("maxwidth", "350px");
      popup.setAttribute("noautofocus", "true");
      popup.setAttribute("ignorekeys", "true");
      popup.setAttribute("allowevents", "true");
      popupset.appendChild(popup);
      
      if (properties.suggestionUrl)
        searchboxes[id]["suggestionUrl"] = properties.suggestionUrl;
      else
        searchboxes[id]["suggestionUrl"] = null;

      if (properties.defaultText)
      {
        searchboxes[id]["defaultText"] = properties.defaultText;
        var defaultText = properties.defaultText;
        if (toolbarchild.value == "")
        {
          toolbarchild.value = defaultText;
          loadContentHelp.onDefaultText(toolbarchild);
          //toolbarchild.style.color = "#CCCCCC";
          searchboxes[id]["isDefaultText"] = true;

        }
      }
        
      toolbarchild.addEventListener("focus", function () {  
        if (searchboxes[id]["defaultText"])
        {
          if (searchboxes[id]["isDefaultText"] == true)
          {
            toolbarchild.value = "";
          }
          searchboxes[id]["isDefaultText"] = false;
          loadContentHelp.offDefaultText(toolbarchild);
          //toolbarchild.style.color = "";
        }
        
        if (searchboxes[id]["suggestionUrl"])
        {
          if (toolbarchild.value == "")
            loadContentHelp.showpopuphelp(toolbarchild, id + '-popup', searchboxes["_text"]["searchSuggestionText"]);
          else
            loadContentHelp.showpopupsuggestion(toolbarchild, id + '-popup', searchboxes[id]["suggestionUrl"]);
        }
      }, false);
        
      toolbarchild.addEventListener("click", function (event) { 
        if (event.target != toolbarchild)
          return;
        if (searchboxes[id]["suggestionUrl"])
        {
          var popup = document.getElementById(id + '-popup');
          popup.openPopup(toolbarchild, "after_start", 0, 0, true, false);
        }
      }, false);

      toolbarchild.addEventListener("input", function (event) {
        if (searchboxes[id]["suggestionUrl"])
        {
          searchboxes[id]["index"] = -1;
          loadContentHelp.searchCurrentTime = loadContentHelp.getTime();

          if (trim(toolbarchild.value + event.charCode) == "")
            loadContentHelp.showpopuphelp(toolbarchild, id + '-popup', searchboxes["_text"]["searchSuggestionText"]);
          else {
            var tmp = function () {
              var t = loadContentHelp.getTime();
              if (t >= (loadContentHelp.searchTimeout + loadContentHelp.searchCurrentTime))
              {
                if (searchboxes[id].showWindow)
                  loadContentHelp.showpopupsuggestion(toolbarchild, id + '-popup', searchboxes[id]["suggestionUrl"]);
              } else
                window.setTimeout(tmp, loadContentHelp.searchTimeout + loadContentHelp.searchCurrentTime - t );
            };
            window.setTimeout(tmp, loadContentHelp.searchTimeout);
          }
        }
      }, false);
      toolbarchild.addEventListener("keypress", function (event) {
        if (event.keyCode == event.DOM_VK_RETURN)
        {
          var popup = document.getElementById(id + '-popup');
          var length = popup.childNodes.length;
          //trace("return:" + searchboxes[id]["index"])
          //trace("return:" + length)
          searchboxes[id].showWindow = false;
          let iid = id;
          var func = function () { searchboxes[iid].showWindow = true; };
          window.setTimeout(func, loadContentHelp.searchTimeout);

          if (searchboxes[id]["index"] >= 0 && searchboxes[id]["index"] < length)
          {
            var child = popup.childNodes[searchboxes[id]["index"]];
            child.oncommand();
            popup.hidePopup();
            return;
          } else {
            if (popup)
              popup.hidePopup();
          }

          ALX_NS_PH_TB.loadSearchURL(event, sq_nostr(searchboxes[id].url), sq_nostr(searchboxes[id].openIn), id);
          return;
        }

        if (event.keyCode == 27)
        {
          searchboxes[id]["index"] = -1;
          var popup = document.getElementById(id + '-popup');
          popup.hidePopup();
          return;
        }

        if (event.keyCode == 37 || event.keyCode == 39)
        {
          searchboxes[id]["index"] = -1;
          return;
        }

        if (event.keyCode == 38)
        {
          var popup = document.getElementById(id + '-popup');
          searchboxes[id]["index"]--;

          loadContentHelp.activeMenuItem(popup, searchboxes[id]);
          return;
        }

        if (event.keyCode == 40)
        {
          var popup = document.getElementById(id + '-popup');
          searchboxes[id]["index"]++;

          loadContentHelp.activeMenuItem(popup, searchboxes[id]);
          return;
        }
      }, false);

      toolbarchild.addEventListener("blur", function () {
        if (searchboxes[id]["defaultText"])
        {
          if (toolbarchild.value.replace(/\s/gi,"") == "")
          {
            toolbarchild.value = searchboxes[id]["defaultText"];
            //toolbarchild.style.color = "#CCCCCC";
            loadContentHelp.onDefaultText(toolbarchild);
            searchboxes[id]["isDefaultText"] = true;
          }
        }
      }, false);
        
      //toolbarchild.setAttribute("id", "ALX_NS_PH_TB-Search-Edit");

      searchboxes["previousSearchbox"] = id;
      //toolbarchild.setAttribute("onkeypress", "if (event.keyCode == event.DOM_VK_RETURN) { ALX_NS_PH_TB.loadSearchURL(event, " + sq(properties.url) + ", " + sq(openIn) + ", '" + id + "'); } ");
    }

    if (menupopup && toolbarchild)
    {
      if (properties.numHeadlines)
      {
        toolbarchild.style.setProperty("padding-right", "1px", "important");
      }
      else
      {
        toolbarchild.style.setProperty("padding-right", "4px", "important");
      }
    }

    if (properties.type == "javascript" && properties.runat && properties.runat == "content")
    {
      loadContentHelp.addSepar(toolbaritem);
      var javascriptContent = "";

      properties.javascriptUrl = subst_tokens(properties.javascriptUrl); 
      if (properties.javascriptUrl && loadContentHelp.validContentUrl(properties.javascriptUrl))
      {
        javascriptContent = "var s=window.document.createElement('script');s.setAttribute('src', '" +properties.javascriptUrl+"');window.document.body.appendChild(s);"
      }

      
      if (properties.trigger && properties.trigger == "click")
      {
        properties._oncommand = function() {
          ALX_NS_PH_TB.log(id + ':c');
          var currentWindow = window.content;
          
          var sandbox = new Components.utils.Sandbox(currentWindow.wrappedJSObject);
          sandbox.window = currentWindow.wrappedJSObject;
          sandbox.__proto__ = currentWindow.wrappedJSObject;
          Components.utils.evalInSandbox(javascriptContent, sandbox);
        };
      }
      else if (properties.trigger && properties.trigger == "load")
        window.content.document.body.appendChild(s);
        
    }
    else if (properties.type == "javascript" && properties.javascript)
    {
      var wrappedWindow = new XPCNativeWrapper(appSvc.hiddenDOMWindow);
      var sandbox = new Components.utils.Sandbox(wrappedWindow);
      var xmlhttpRequester = new GM_xmlhttpRequester();
      sandbox.window = wrappedWindow;
      sandbox.document = wrappedWindow.document;
      sandbox.navigator = wrappedWindow.navigator;
      sandbox.__proto__ = wrappedWindow;
      var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

      sandbox.Alexa = {
        browser: {
          arguments: null,
          getLocale: function()
          {
            var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
            var locale = "default";
            if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocale") )
              locale = prefBranch.getCharPref("extensions.alexa.toolbarLocale");
            return locale;
          },
          setLocale: function( str )
          {
            if (str)
            {
              var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
              prefBranch.setCharPref("extensions.alexa.toolbarLocale", str);
            }
          },
          navigate: function(url, openIn, features, args, event)
          {
            if (!event)
            {
              event = {target: toolbarchild};
            }
            trace("i am here: " + url)
            var win = ALX_NS_PH_TB.loadURL(event, url, openIn, features, args);
            if (win)
            {
              var winWrapper = {
                close:  function() { return win.close() },
                isClosed: function() { return win.closed },
                focus: function() { return win.focus() },
                isVisible: function() { return win.state == "open" },
                show: function() { return win.show() },
                hide: function() { return win.hide() }
              };
              return winWrapper;
            }
          },
          getSiteInfo: get_data,
          getSelectedText: function()
          {
            try
            {
              return window.content.getSelection().toString();
            }
            catch (ex)
            {
              return "";
            }
          },
        },
        button: {
          hide: function() { toolbarchild.style.display = "none"; },
          show: function() { toolbarchild.style.display = "";  },
          update: function()
          {
            ALX_NS_PH_TB.updateButtons(get_data());
          },
          properties: properties,
          transform: function( transString )
          {
            toolbarchild.style.MozTransform = transString;
          },
          images: {
            clear: function()
            {
              var nextItem = null;
              for (var item = toolbaritem.firstChild; item != null; item = nextItem) {
                nextItem = item.nextSibling;
                if (item.nodeName == "image")
                {
                  toolbaritem.removeChild(item);
                }
              }
            },
            add: function(attributes, events, isOffSeparator)
            {
              var image = document.createElement("image");
              for (var key in attributes)
              {
                image.setAttribute(key, String(attributes[key]));
              }
              for (var key in events)
              {
                image.addEventListener(key, events[key], false);
              }
              toolbaritem.appendChild(image);
              if (!isOffSeparator)
              {
                loadContentHelp.addSepar(toolbaritem);
              }
            }
          },
          menu: {
            create: function()
            {
              if (!menupopup)
              {
                menupopup = document.createElement("menupopup");
                menupopup.addEventListener("popupshowing", function(event)
                {
                  ALX_NS_PH_TB.log(id + ":m");
                  if (properties.updated)
                  {
                    properties.updated = false;
                    ALX_NS_PH_TB.updateButtons(get_data());
                  }
                }, false);
                toolbarchild.appendChild(menupopup);
              }
              toolbarchild.setAttribute("type", "menu-button");
              if (properties.numHeadlines)
              {
                toolbarchild.style.setProperty("padding-right", "1px", "important");
              }
              else
              {
                toolbarchild.style.setProperty("padding-right", "4px", "important");
              }
            },
            destroy: function()
            {
              if (menupopup)
              {
                ALX_NS_PH_SPARKLINE.clearMenu(menupopup);
                toolbarchild.setAttribute("type", "button");
              }
            },
            clear: function()
            {
              ALX_NS_PH_SPARKLINE.clearMenu(menupopup);
            },
            addSeparator: function()
            {
              var menuseparator = document.createElement("menuseparator");
              menupopup.appendChild(menuseparator);
            },
            addTextItem: function(label)
            {
              var menuitem = document.createElement("menuitem");
              menuitem.setAttribute("label", label);
              menuitem.oncommand = function(event) {
                event.stopPropagation();
              };
              menupopup.appendChild(menuitem);
            },
            addCallbackItem: function(label, callback, cookie, icon)
            {
              var menuitem = document.createElement("menuitem");
              menuitem.setAttribute("tooltiptext", label);
              menuitem.setAttribute("label", label);
              menuitem.oncommand = function(event) {
                ALX_NS_PH_TB.log(id + ':i');
                callback(event, cookie);
                event.stopPropagation();
              };
              menuitem.addEventListener("command", menuitem.oncommand, false);
              if (icon)
              {
                menuitem.setAttribute("class","menuitem-iconic alexa-favicon");
                menuitem.setAttribute("image",icon);
              }
              menupopup.appendChild(menuitem);
            },
            addUrlItem: function(label, callback, openIn, windowFeatures, icon)
            {
              var menuitem = document.createElement("menuitem");
              menuitem.setAttribute("tooltiptext", label);
              menuitem.setAttribute("label", label);
              menuitem.oncommand = function(event) {
                ALX_NS_PH_TB.log(id + ':i');
                ALX_NS_PH_TB.loadURL(event, callback, openIn, windowFeatures);
                event.stopPropagation();
              };
              menuitem.addEventListener("command", menuitem.oncommand, false);
              if (icon)
              {
                menuitem.setAttribute("class","menuitem-iconic alexa-favicon");
                menuitem.setAttribute("image",icon);
              }
              menupopup.appendChild(menuitem);
            }
          },
        },
        events: {
          preventDefault: function(event) { event.preventDefault()}, 
          addPopupShowingListener: function(callback) {
            menupopup.addEventListener("popupshowing", callback, false);
          },
          addPopupHidingListener: function(callback) {
            menupopup.addEventListener("popuphiding", callback, false);
          },
          addListener: function(target, type, listener, useCapture) {
            target.addEventListener(type, listener, useCapture);
          },
          addPageTurnListener: function(callback)
          {
            pageturn_listeners.push(callback);
          },
          addLeftClickListener: function(callback)
          {
            properties._oncommand = function(event)
            {
              ALX_NS_PH_TB.log(id + ':c');
              callback({clientX: event.clientX, clientY: event.clientY, preventDefault: function() { event.preventDefault(); } });
            };
          },
          addSaveClientStateListener: function(key, callback)
          {
             var branch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
             var key_prefix = "extensions.alexa.active-buttons." + id + ".storage.";
             var long_key = key_prefix + key;
             var observer = {
               observe: function(subject, topic, data)
               {
                 if (topic == "nsPref:changed")
                 {
                   callback(data.replace(key_prefix, ""), branch.getCharPref(data));
                 }
               },
               shutdown: function()
               {
                 branch.removeObserver(long_key, this);
               }
             };
             branch.addObserver(long_key, observer, false);
             observers.push(observer);
          },
        },
        storage: {
          saveSessionState: function(key, value)
          {
            environment.set("extensions.alexa.active-buttons." + id + ".storage." + key, value);
          },
          readSessionState: function(key, value)
          {
            var full_key = "extensions.alexa.active-buttons." + id + ".storage." + key;
            if (environment.exists(full_key))
            {
              return environment.get(full_key);
            }
            return "";
          },
          saveClientState: function(key, value)
          {
            prefBranch.setCharPref("extensions.alexa.active-buttons." + id + ".storage." + key, value);
          },
          readClientState: function(key)
          {
            try
            {
              var result = prefBranch.getCharPref("extensions.alexa.active-buttons." + id + ".storage." + key);
              return String(result);
            }
            catch (ex)
            {
              return "";
            }
          }
        },
        tools: {
          XMLHttpRequest: GM_hitch(xmlhttpRequester, "contentStartRequest"),
          sandboxRequest: sandboxedRequest,
          JSON: {
            decode: function(str) { return nativeJSON.decode(str); },
            encode: function(obj) { return nativeJSON.encode(obj); }
          },
          initFeedbutton: function(refreshFunc, p) {
            var prop = properties;
            var rFunc = ALX_NS_PH_TB.rss.fillRSSButton;
            if (p)
              prop = p;
            if (refreshFunc)
              rFunc = refreshFunc;
            
            //trace(rFunc);
            toolbarchild.setAttribute("type", "menu-button");
            menupopup = document.createElement("menupopup");
            ALX_NS_PH_TB.rss.initRSSButton(toolbaritem, toolbarchild, prop, menupopup, rFunc);
            
          },
          getFeedLoader: function (refreshFunc, p) {
            var prop = properties;
            var rFunc = ALX_NS_PH_TB.rss.fillRSSButton;
            if (p)
              prop = p;
            if (refreshFunc)
              rFunc = refreshFunc;

            if (!feedwrappers[prop.id])
            {
              feedwrappers[prop.id] = new feedWrapper(prop, rFunc);
            }
            else
            {
              feedwrappers[prop.id].properties = prop;
              feedwrappers[prop.id].refreshFunc = rFunc;
            }
            return feedwrappers[prop.id];
          },
          DOMParser: appSvc.hiddenDOMWindow.DOMParser,
          getNodeText: function(node) { return node.textContent; },
          setNodeText: function(node, text) { node.textContent = text; },
          trace: trace
        },
      };
      sandbox.setTimeout = function(callback, timeout)
      {
        var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        if (typeof callback == "function")
        {
          timer.initWithCallback({notify: callback}, timeout, Ci.nsITimer.TYPE_ONE_SHOT);
        }
        else
        {
          timer.initWithCallback({notify: function() { Components.utils.evalInSandbox(callback, sandbox); } }, timeout, Ci.nsITimer.TYPE_ONE_SHOT);
        }
        timers.push(timer);
      };
      sandbox.setInterval = function(callback, timeout)
      {
        var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        if (typeof callback == "function")
        {
          timer.initWithCallback({notify: callback}, timeout, Ci.nsITimer.TYPE_REPEATING_SLACK);
        }
        else
        {
          timer.initWithCallback({notify: function() { Components.utils.evalInSandbox(callback, sandbox); } }, timeout, Ci.nsITimer.TYPE_REPEATING_SLACK);
        }
        timers.push(timer);
      };
      sandbox.trace = trace;

      trace("to see if i am here 2")
      trace(properties.id)
      Components.utils.evalInSandbox(properties.javascript, sandbox, "1.6", filename + "#" + id, 1);
    }

    toolbar_buttons[properties.id] = {"child": toolbarchild, "properties": properties};
  }

  function subst_tokens(url, data)
  {
    if (!url)
    {
      return url;
    }
    var rankText = "no data";
    var rankTip = "no data";
    var averageReview = "unrated";
    var href = "";
    var site = "";
    if (data)
    {
      rankText = data.rankText;
      rankTip = data.rankTip;
      site = data.site;
    }

    try
    {
      href = window.content.document.location.href;
    }
    catch (ex)
    {
    }

    url = url.replace(/__RANK__TEXT__PLACEHOLDER__/g, rankText);
    url = url.replace(/__RANK__TIP__PLACEHOLDER__/g, rankTip);

    try {
      url = url.replace(/__VERSION__PLACEHOLDER__/g, alexaUtilityService.getMyVersion());
    } catch (ex) {
      try {
        url = url.replace(/__VERSION__PLACEHOLDER__/g, "");
      } catch (ex) {
      }
    }

    try {
      url = url.replace(/__SELECTION__PLACEHOLDER__/g, window.content.window.getSelection());
    }
    catch (ex) {
      try {
        url = url.replace(/__SELECTION__PLACEHOLDER__/g, "");
      } catch (ex) {

      }
    }

    if (loadContentHelp.isJsorXml(url)) 
    {
      url = url.replace(/__VERSION__TOOLBAR__ROOT__PLACEHOLDER__/g, "chrome://ALX_NS_PH/content");
      url = url.replace(/__TOOLBAR__ROOT__PLACEHOLDER__/g, "chrome://ALX_NS_PH/content");
    } else {
      url = url.replace(/__VERSION__TOOLBAR__ROOT__PLACEHOLDER__/g, "chrome://ALX_NS_PH/skin");
      url = url.replace(/__TOOLBAR__ROOT__PLACEHOLDER__/g, "chrome://ALX_NS_PH/skin");
    }
    
    url = url.replace(/__SITE__PLACEHOLDER__/g, site);
    url = url.replace(/__ENCODED__URL__PLACEHOLDER__/g, encodeURIComponent(href));
    url = url.replace(/__LOCATION__BAR__PLACEHOLDER__/g, encodeURIComponent(href));
    url = url.replace(/__URL__PLACEHOLDER__/g, href);
    url = url.replace(/__TOOLBAR__LOCALE__BASE__PLACEHOLDER__/g, "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/locale");
    url = url.replace(/__BROWSER__PLACEHOLDER__/g, "f");

    for (var key in toolbar_global)
      url = url.replace(new RegExp(key, "g"), toolbar_global[key]);

    return url;
  }

  function get_toolbar_global_configure(toolbarxml)
  {
    var configureNode  = null;
    var configureNodes = toolbarxml.documentElement.getElementsByTagName("configuration");
    if ( configureNodes )
      configureNode = configureNodes[0];

    if ( configureNode )
    {
      var children = configureNode.childNodes;
      for (var i = 0; i < children.length; i++)
      {
        var item = children.item(i);
        var tagName = item.tagName;
        switch (tagName)
        {
          case "pluginReportingIdentifier":
          case "regionPlaceHolder":
          case "tagNPlaceHolder":
          case "postInstallBubbleShown":
            var localeId  = item.getAttribute("id");
            toolbar_global[tagName] = loadContentHelp.getLocalNameConfigure(item.textContent, localeId);
            break;
        }
      }
    }

    var replacementsNode  = null
    var replacementsNodes = toolbarxml.documentElement.getElementsByTagName("replacements");

    if ( replacementsNodes )
      replacementsNode = replacementsNodes[0];
      
    if ( replacementsNode )
    {
      var replacementNodes = replacementsNode.getElementsByTagName("replacement");
      for (var i = 0; i < replacementNodes.length; i++)
      {
        var replacementNode = replacementNodes[i];
        var keyNodes        = replacementNode.getElementsByTagName("key");
        var valueNodes      = replacementNode.getElementsByTagName("value");
        
        for (var j = 0; j < keyNodes.length; j++)
        {
          var key   = keyNodes[j].textContent;
          var value = valueNodes[j].textContent
          toolbar_global[key] = loadContentHelp.getLocalNameConfigure(value, key);
        }
      }
    }
  }

  function add_buttons_from_toolbarxml(toolbarxml, src, cb)
  {
    var toolbar = document.getElementById("ALX_NS_PH_TB-Toolbar");
    // Delete all buttons from toolbar in case this function
    // got called twice in a row (from two buttons being added at once).
    // NOTE: This doesn't completely solve the problem!
    while (toolbar.firstChild) {
      toolbar.removeChild(toolbar.firstChild);
    }
    toolbar_buttons = {};
    toolbar_global  = {};
    get_toolbar_global_configure(toolbarxml);
    
    var toolbarspring = document.createElement("toolbarspring");
    toolbarspring.setAttribute("id", "ALX_NS_PH_TB-Toolbar-Spring");
    toolbarspring.setAttribute("minwidth", "30");
    toolbar.appendChild(toolbarspring);
    var toolbarpopupset = document.createElement("popupset");
    toolbarpopupset.setAttribute("id", "ALX_NS_PH_TB-Toolbar-popupset");
    toolbar.appendChild(toolbarpopupset);
    
    var buttons = toolbarxml.documentElement.getElementsByTagName("button");
    if ( buttons )
    for ( var i = 0 ; i < buttons.length ; ++i )
    { 
      var button = buttons.item(i);
      add_button(toolbar, button, src);
    }
    cb();
    init_in_progress = false;
  }

  function add_buttons(src, cb)
  {
    var toolbar = document.getElementById("ALX_NS_PH_TB-Toolbar");
    var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
    var toolbarXMLText = null;
    var ver = alexaUtilityService.getMyVersion();

    if (prefBranch.prefHasUserValue("extensions.alexa.toolbarXMLText") && 
        prefBranch.prefHasUserValue("extensions.alexa.toolbarXMLVersion") &&
        prefBranch.getCharPref("extensions.alexa.toolbarXMLVersion") == ver)
    {
      toolbarXMLText = decodeURIComponent(escape(prefBranch.getCharPref("extensions.alexa.toolbarXMLText")));
    }

    if (toolbarXMLText)
    {
      var dp = new DOMParser();
      var doc = dp.parseFromString(toolbarXMLText, "text/xml");
      add_buttons_from_toolbarxml(doc, src, cb)
    } else {
      //trace("have no toolbarXMLText");
      var http = new XMLHttpRequest();
      http.open( "GET", src, true );
      http.onload = function()
      {
        add_buttons_from_toolbarxml(http.responseXML, src, cb)
      };
      http.send(null);
    }
  }

  function get_data()
  {
    var data = sessionService.getTabValue( getBrowser().selectedTab, "ALX_NS_PH.data" );
    if (data)
      data = nativeJSON.decode(data);
    return data;
  }

  function init_toolbar()
  {
    // destroy toolbar first
    ALX_NS_PH_TB.extension.shutdown();
    
    // write a callback function for add_button function
    var add_button_callback = function()
    {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   . getService(Components.interfaces.nsIPrefService);
      var branch = prefs.getBranch("extensions.alexa.");
      var active_buttons = branch.getCharPref("active-buttons").split(",");
      var toolbar = document.getElementById("ALX_NS_PH_TB-Toolbar");
      for (var i = 0; i < active_buttons.length; i++)
      {
        var key = active_buttons[i];
        if (key && key.length)
        {
          var ver = "";
          var filename = "";
          try
          {
            filename = branch.getCharPref("active-buttons." + key + ".url");
            ver = branch.getCharPref("active-buttons." + key + ".version");
          }
          catch (ex)
          {
          }
          if (filename && ver != alexaUtilityService.getMyVersion())
          {
            ALX_NS_PH_TB.register_buttons("ALX_NS_PH" , filename, true);
          }

          var xml = branch.getCharPref("active-buttons." + key);
          var parser = new DOMParser();
          var doc = parser.parseFromString(xml, "text/xml");
          var buttons = doc.documentElement.getElementsByTagName("button");
          if ( buttons )
          for ( var j = 0 ; j < buttons.length ; ++j )
          {
            var button = buttons.item(j);
            trace("to see if i am here")
            add_button(toolbar, button, filename);
          }
        }
      }
      ALX_NS_PH_TB.updateButtons(get_data());     
    };
    
    loadContentHelp.loadLocale(); 
    add_buttons("chrome://ALX_NS_PH/content/toolbar.xml", add_button_callback);
    
  }

  var alexaUtilityService = Components.classes["@ALX_NS_PH/utility-service;1"].createInstance( Components.interfaces.nsIAlexaUtilityService );
  if (typeof ALX_NS_PH_TB == "undefined") {
    window.ALX_NS_PH_TB = {
      ID: "ALX_NS_PH_TB@alexa.com",
      INSTALL_SERVER: "http://centy.lottasoft.com/wizard",

      getFeatures: function()
      {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
             . getService(Components.interfaces.nsIPrefService);
        var branch = prefs.getBranch("extensions.alexa.");
        var value = null;
        var params = [];
        if ( branch.getBoolPref("reviews") )
          params.push("reviews");
        if ( branch.getBoolPref("ranks") )
          params.push("website_rankings");
        if ( branch.getBoolPref("searchranks") )
          params.push("search_rankings");
        if ( branch.getBoolPref("error-handling") )
          params.push("error_handling");
        if ( branch.getBoolPref("location-bar-search") )
          params.push("location_bar_search");
        if ( branch.getBoolPref("usage-stats") )
          params.push("usage_stats");
        if ( branch.getBoolPref("alexa-relatedlinks") )
          params.push("alexa_relatedlinks");

        var dispStyle = "display_style_" + branch.getCharPref("display-style");
        params.push(dispStyle);
        
        return params;
        
      },
      configure: function(params, isPostBubble)
      {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
             . getService(Components.interfaces.nsIPrefService);
        var branch = prefs.getBranch("extensions.alexa.");

        for (key in params)
        {
          if (params[key] == "not-checked")
            params[key] = false;
          if (params[key] == "checked")
            params[key] = true;
          if (key == "website_rankings")
            branch.setBoolPref("ranks", params[key]);
          else if (key == "search_rankings")
            branch.setBoolPref("searchranks", params[key]);
          else if (key == "usage_stats")
            branch.setBoolPref("usage-stats", params[key]);
          else if (key == "location_bar_search")
            branch.setBoolPref("location-bar-search", params[key]);
          else if (key == "error_handling")
            branch.setBoolPref("error-handling", params[key]);
          else if (key == "alexa_relatedlinks")
            branch.setBoolPref("alexa-relatedlinks", params[key]);
          else if (key == "display_style")
            branch.setCharPref("display-style", params[key]);
          else
            branch.setBoolPref(key, params[key]);
        }
        
        if (params.location_bar_search)
          branch.setBoolPref('location-bar-search', params.location_bar_search);
        if (params.error_handling)
          branch.setBoolPref('error-handling', params.error_handling);
        
        if ( isPostBubble )
          alexaUtilityService.ping(true);
        if (configure_window)
        {
          branch.setBoolPref('configured2', 1);
          configure_window.hide();
        }
      },
      
      popup_configuration: function()
      {
        loadContentHelp.loadLocale();
        if (toolbar_global["postInstallBubbleShown"] && toolbar_global["postInstallBubbleShown"] != "true")
        {
          alexaUtilityService.ping(true);
          var prefs = Components.classes["@mozilla.org/preferences-service;1"]
             . getService(Components.interfaces.nsIPrefService);
          var branch = prefs.getBranch("extensions.alexa.");
          branch.setBoolPref('configured2', 1); 
        }
        else
          configure_window = ALX_NS_PH_TB.loadURL({target: logo}, "__TOOLBAR__LOCALE__BASE__PLACEHOLDER__/" + loadContentHelp.currentLocale + "/bubbles/AfterInstall.htm", "newDialog", "width=600,height=300,transparent,noautohide,align=after_start,left=7,top=1");
      },

      getOS: function() {
        return Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;
      },
      getExtensionManager: function() {
        return Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
      },
      isWindows: function() {
        return "WINNT" == ALX_NS_PH_TB.getOS();
      },
      isMac: function() {
        return "Darwin" == ALX_NS_PH_TB.getOS();
      },
      isLinux: function() {
        return "Linux" == ALX_NS_PH_TB.getOS();
      },
      isToolbarVisible : function() {
        var rect = document.getElementById("ALX_NS_PH_TB-Toolbar").getClientRects()[0];
        return (rect.top != rect.bottom);
      },
      getHost: function(doc) {
        try {
          return doc.location.host;
        } catch (ex) {
          return "";
        }
      },
      getHref: function(doc) {
        try {
          return doc.location.href;
        } catch (ex) {
          return "";
        }
      },
      internalLoadURL: function(url, openIn, features, args, event) {
        url = subst_tokens(url, get_data());

        if ("newTab" == openIn) {
          gBrowser.selectedTab = gBrowser.addTab(url);
        } else if ("newWindow" == openIn) {
          window.open(url);
        } else if ("sidebar" == openIn) {
          toggleSidebar("ALX_NS_PH_TB_viewEmptySidebar", true);
          window.sidebar.location.href = url;
        } else if ("newDialog" == openIn) {
          var external = {
            'navigate': ALX_NS_PH_TB.public_loadURL,
            'register_buttons': ALX_NS_PH_TB.register_buttons,
            'trace': trace,
            'ALX_NS_PH_TB': 1
          };


          var featureArray = features.split(",");
          var validFeatures = [];
          var height, width, hidden = 0, background = 0, transparent = 0, left = 0, top = 0, noautohide = 0, align = "after_end", resizable = 1, status = 1;
          for (var i = 0; i < featureArray.length; i++)
          {
            var feature = featureArray[i];
            if ( feature.match("^alexaapi$") )
            {
              external['getFeatures'] = ALX_NS_PH_TB.getFeatures;
              external['configure'] = ALX_NS_PH_TB.configure;
              external['jsonEncode'] = function(obj) { return nativeJSON.encode(obj); };
              external['getLocale'] = function() {
                var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
                var locale = "default";
                if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocale") )
                  locale = prefBranch.getCharPref("extensions.alexa.toolbarLocale");
                return locale;
              };
            }
            else if (feature.match("^height="))
            {
              height = feature.replace(/^height=/,"");
              validFeatures.push(feature);
            }
            else if (feature.match("^width="))
            {
              width = feature.replace(/^width=/,"");
              validFeatures.push(feature);
            }
            else if (feature.match("^transparent(=yes)?$"))
            {
              transparent = 1;
            }
            else if (feature.match("^hidden(=yes)?$"))
            {
              hidden = 1;
            }
            else if (feature.match("^background(=yes)?$"))
            {
              background = 1;
            }
            else if (feature.match("^noautohide(=yes)?$"))
            {
              noautohide = 1;
            }
            else if (feature.match("^resizable=no$"))
            {
              resizable = 0;
            }
            else if (feature.match("^status=no$"))
            {
              status = 0;
            }
            else if (feature.match("^align="))
            {
              align = feature.replace(/^align=/,"");
            }
            else if (feature.match("^left="))
            {
              left = parseInt(feature.replace("left=",""), 10);
            }
            else if (feature.match("^top="))
            {
              top = parseInt(feature.replace("top=",""), 10);
            }
            else
            {
              validFeatures.push(feature);
            }
          }
          var dialog = null;
          if (hidden)
          {
             if (iframes[url])
             {
               trace("Error: Script opened same hidden URL twice without closing the first URL");
               return;
             }
             var hiddenDocument = appSvc.hiddenDOMWindow.document;
             var iframe = hiddenDocument.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul","iframe");
             iframe.setAttribute("src", url);
             iframe.setAttribute("type", "content");
             hiddenDocument.documentElement.appendChild(iframe);
             iframe.addEventListener("DOMContentLoaded", function (event) {
               if (event.originalTarget instanceof HTMLDocument)
               {
                 if (!background && typeof noscriptOverlay != 'undefined')
                 {
                   var ns = noscriptOverlay.ns;
                   var site = ns.getQuickSite(url, 3);
                   var blocked = false;
                   var unblock_list = [];
                   if (!ns.isJSEnabled(site))
                   {
                     blocked = true;
                     unblock_list.push(site);
                   }
                   var doc = event.originalTarget;
                   var scripts = doc.getElementsByTagName("script");
                   for (var i = 0; i < scripts.length; i++)
                   {
                     if (scripts[i].src)
                     {
                       site = ns.getQuickSite(scripts[i].src, 3);
                       if (!ns.isJSEnabled(site))
                       {
                         if (unblock_list.indexOf(site) == -1)
                         {
                           unblock_list.push(site);
                         }
                         blocked = true;
                       }
                     }
                   }

                   if (blocked)
                   {
                     // NOTE: This text is currently specialized for the
                     // Facebook button because it's the only button which
                     // uses hidden windows right now.
                     var title = "Facebook Button: Allow Javascript?";
                     var content = "The Facebook Button uses Javascript to download information\nabout your friends from Facebook. "
                     content += "For this button to work,\nthe toolbar needs to enable Javascript on the following sites:\n\n";
                     content += "  " + unblock_list.join(", ");
                     content += "\n\nAllow Javascript to run on these sites?";
                     if (promptSvc.confirm(null, title, content))
                     {
                       for (var i = 0; i < unblock_list.length; i++)
                       {
                         ns.setJSEnabled(unblock_list[i], true, false, true);
                       }
                       event.originalTarget.defaultView.location.reload();
                     }
                   }
                 }

                 if (event.originalTarget.wrappedJSObject)
                 {
                   // Pass on our arguments
                   event.originalTarget.wrappedJSObject.defaultView.arguments = [ external, args ];

                   // Give the hidden window privileges to launch popup windows
                   // (E.g. for the "facebook login" window)
                   event.originalTarget.wrappedJSObject.defaultView.open = function(url, name, features)
                   { 
                     var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
                     var opener = event.originalTarget.defaultView;
                     var win = ww.openWindow(opener, url, name, features, null);
                     return win.wrappedJSObject;
                   }
                 }
                 else
                 {
                   event.originalTarget.defaultView.arguments = [ external, args ];
                 }
               }
             }, true);
             dialog = {
               close: function() { if (iframe) { if (iframe.parentNode) iframe.parentNode.removeChild(iframe); delete iframes[url]; iframe = null; } },
             };
             iframes[url] = dialog;
          }
          else if (url && !url.match("^chrome://") || url && transparent)
          {
            var correction = -5;
            var statusbar = document.getElementById("status-bar");
            if (statusbar && statusbar.getBoundingClientRect().top)
            {
              correction += statusbar.getBoundingClientRect().top - statusbar.getBoundingClientRect().bottom;
            }
            var guessY = window.content.screenY + window.content.outerHeight - window.content.innerHeight + correction;
            var guessX = window.content.screenX;
            if (event && event.target)
            {
              guessX = guessX + event.target.getBoundingClientRect().right - width + 20;
            }
            validFeatures.push("top=" + (guessY + top))
            validFeatures.push("left=" + (guessX + left))
            validFeatures.push("dependent");
            validFeatures.push("chrome=no");

            if (resizable)
            {
              validFeatures.push("resizable");
            }
            if (status)
            {
              validFeatures.push("status");
            }
            if (!transparent)
            {
              dialog = window.openDialog(url, "_blank", validFeatures.join(","), external, args);
            }
            else
            {
              if (popup_panels[url] && !popup_panels[url].closed)
              {
                popup_panels[url].hide();
                popup_panels[url].init(event, left, top, args);
                popup_panels[url].show();
                return popup_panels[url];
              }

              var panels = document.getElementsByTagName("panel");
              for (var i = 0; i < panels.length; i++)
              {
                if (panels[i].getAttribute("url") == url)
                {
                  panels[i].parentNode.removeChild(panels[i]);
                  break;
                }
              }
                
              var panel = document.createElement("panel");
              if (width) panel.style.width = width;
              if (height) panel.style.height = height;
              panel.style.background = "transparent";
              panel.style.border = "0px solid transparent";
              panel.setAttribute("url", url);
              if (noautohide)
              {
                panel.setAttribute("noautohide", "true");
              }

              // Bug 3789: On mac twitter box and alexa configure box have
              // a white outline around them.
              panel.style.MozAppearance = "none";

              dialog = {
                init: function(_event, _left, _top, _args) { event = _event; left = _left; top = _top; args = _args; },
                show: function() { if (panel && panel.parentNode) panel.openPopup(event.target, align, left, top, false, true) },
                hide: function() { if (panel && panel.parentNode) panel.hidePopup(); },
                close: function() {
                  if (panel) {
                    if (panel.parentNode) {
                      panel.hidePopup();
                      panel.parentNode.removeChild(panel);
                    }
                    delete popup_panels[url];
                    panel = null;
                    dialog = null;
                  }
                },
                panel: panel
              };
              dialog.__defineGetter__("closed", function() { return !(panel && panel.parentNode); });
              dialog.__defineGetter__("state", function() { return panel ? panel.state : null });

              var http = new XMLHttpRequest();
              http.overrideMimeType("text/xml");
              http.open( "GET", url, true );
              http.onload = function()
              {
                var text = http.responseText;
                text = text.replace(/\s/gi, "");
                if (text == "")
                {
                  dialog.close();
                  return;
                }   
                var xml = http.responseXML;
                var wrappedWindow = new XPCNativeWrapper(window);
                wrappedWindow.close = function() { dialog.close(); };
                var sandbox = new Components.utils.Sandbox(wrappedWindow);
                sandbox.window = wrappedWindow;
                sandbox.document = new XPCNativeWrapper(document);
                sandbox.trace = trace;
                sandbox.Alexa = {
                  browser: {
                    arguments: function() { return args },
                    registerButtons: ALX_NS_PH_TB.register_buttons,
                    configure: ALX_NS_PH_TB.configure,
                    navigate: function(url, openIn, features, args)
                    {
                      var win = ALX_NS_PH_TB.loadURL(event, url, openIn, features, args);
                      if (win)
                      {
                        var winWrapper = {
                          close:  function() { return win.close() },
                          isClosed: function() { return win.closed },
                          focus: function() { return win.focus() },
                          isVisible: function() { return win.state == "open" },
                          show: function() { return win.show() },
                          hide: function() { return win.hide() },
                        };
                        return winWrapper;
                      }
                    },
                    getSiteInfo: get_data,
                    getSelectedText: function()
                    {
                      try
                      {
                        return window.content.getSelection().toString();
                      }
                      catch (ex)
                      {
                        return "";
                      }
                    },
                    reload: function() {
                      dialog.close();
                      ALX_NS_PH_TB.internalLoadURL(url, openIn, features, args, event);
                    }
                  },
                  events: {
                    preventDefault: function(event) { event.preventDefault()}, 
                    addPopupShowingListener: function(callback) {
                      panel.addEventListener("popupshowing", callback, false);
                    },
                    addPopupHidingListener: function(callback) {
                      panel.addEventListener("popuphiding", callback, false);
                    },
                    addListener: function(target, type, listener, useCapture) {
                      target.addEventListener(type, listener, useCapture);
                    }
                  },
                  tools: {
                    getNodeText: function(node) { return node.textContent; },
                    setNodeText: function(node, text) { node.textContent = text; },
                    trace: trace
                  }
                };
                sandbox.__proto__ = wrappedWindow;

                var nodesSnapshot = xml.evaluate('//*', xml, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null );
                for (var i=0 ; i < nodesSnapshot.snapshotLength; i++)
                {
                  var item = nodesSnapshot.snapshotItem(i);
                  var removeList = [];
                  for (var j=0; j < item.attributes.length; j++)
                  {
                    var attrib = item.attributes[j];
                    if (attrib.specified && attrib.name.match(/^on/))
                    {
                      var event_name = attrib.name.replace(/^on/,"");
                      let value = attrib.value;
                      item.addEventListener(event_name, function(event) {
                        sandbox.event = event;
                        Components.utils.evalInSandbox(value, sandbox);
                        sandbox.event = null;
                      }, false);
                      removeList.push(attrib.name);
                    }
                  }
                  for (var j=0; j < removeList.length; j++)
                  {
                    item.removeAttribute(removeList[j]);
                  }
                }
                var scripts = xml.getElementsByTagName("script");
                removeList = [];
                for (var i = 0; i < scripts.length; i++)
                {
                  Components.utils.evalInSandbox(scripts[i].textContent, sandbox, "1.6", url, 1);
                  removeList.push(scripts[i]);
                }
                for (var i = 0; i < removeList.length; i++)
                {
                  removeList[i].parentNode.removeChild(removeList[i])
                }
                var doc = xml.documentElement;
                if (panel)
                {
                  panel.appendChild(doc);
                  if (panel.state == "closed")
                  {
                    panel.openPopup(event.target, align, left, top, false, true);
                  }
                }
              };
              http.send(null);
              
              var popupset = window.document.getElementById('browser-bottombox');
              popupset.appendChild(panel);
              popup_panels[url] = dialog;
            }
          }
          else
          {
            dialog = window.openDialog(url, "_blank", features, external, args);
          }
          return dialog;
        } else {
          window.content.document.location = url;
          window.content.focus();
        }
      },
      alertObject : function(obj) {
        var msg = "";
        try {
          for (var prop in obj) {
            try {
              msg = msg + prop + " : " + obj[prop] + "\n";
            } catch(ex) {

            }
          }
        } catch(ex) {
          msg = "undefined";
        }
        alert(msg);
      },
      public_loadURL : function (url, openIn, features, args) {
        return ALX_NS_PH_TB.internalLoadURL(url, openIn, features, args);
      },
      loadURL: function(event, url, openIn, features, args) {
        return ALX_NS_PH_TB.internalLoadURL(url, openIn, features, args, event);
      },
      updatePopups: function()
      {
        for (var i in popup_panels)
        {
          if (popup_panels[i].state == "open")
          {
            popup_panels[i].hide();
            popup_panels[i].show();            
          }
        }
      },
      checkLocalDefault: function( strLoc )
      {
        var data = get_data();
        return ALX_NS_PH_TB.checkLocal(subst_tokens(strLoc, data), "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar");
      },
      checkLocal: function(localUrl, remoteBase)
      {
        if (!localUrl)
          return;

        var MY_ID = "toolbar@alexa.com";
        var now = new Date(); now = now.getTime();
        var ttl = 86400000; // one day
        if (localUrl.indexOf("chrome://ALX_NS_PH/") == 0)
        {
          let fileUrl = localUrl.replace("chrome://ALX_NS_PH/", "");
          let profD = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile)
          profD.append("extensions");profD.append(MY_ID)
          let fileUrlL = fileUrl.split("/")
          for (var i in fileUrlL)
            profD.append(fileUrlL[i])
          let filePath = profD.path
          let file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
          file.initWithPath(filePath);

          if(!file.exists())
          {
            let fileUrlTmp = localUrl.replace("chrome://ALX_NS_PH/skin/", "");
            fileUrlTmp = fileUrlTmp.replace("chrome://ALX_NS_PH/content/", "");
            let tmpD = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("TmpD", Ci.nsIFile)
            tmpD.append("alexa");tmpD.append(MY_ID);tmpD.append("2.01");tmpD.append("toolbar");
            let fileUrlTmpL = fileUrlTmp.split("/")
            for (var i in fileUrlTmpL)
              tmpD.append(fileUrlTmpL[i])
              
            let filePathTmp = tmpD.path;
            let fileTmp = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
            fileTmp.initWithPath(filePathTmp);

            if (!fileTmp.exists())
            {
              filePath = filePath.replace(/\\/ig, "/");
              var fparts = filePath.split("skin/");
              if (fparts.length == 1)
                fparts = filePath.split("content/");
              var returnUrl = remoteBase + "/" + fparts[1];
              loadContentHelp.downloadFile(returnUrl, filePathTmp);
              return returnUrl;
            }
 
            if (fileTmp.fileSize == 0 || fileTmp.lastModifiedTime + ttl < now)
            {
              if ( fileTmp.isWritable() )
                fileTmp.remove(false);
              return returnUrl;
            }
            return "file:///" + filePathTmp;
          }
          return localUrl;
        } else if (localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar") == 0 || localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/buttons") == 0)
        {
          let fileUrl = ""; 

          if ( localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar") == 0 )
          {
            fileUrl = "alexa/" + MY_ID + "/2.01/toolbar" + localUrl.replace("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar", "");
          }
          else if ( localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/buttons") == 0 )
          {
            fileUrl = "alexa/" + MY_ID + "/2.01/buttons" + localUrl.replace("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/buttons", "");
          }

          let tmpD = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("TmpD", Ci.nsIFile)
          let fileUrlL = fileUrl.split("/")
          for (var i in fileUrlL)
            tmpD.append(fileUrlL[i])
          let filePath = tmpD.path;
          let file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
          file.initWithPath(filePath);
          if (!file.exists())
          {
            loadContentHelp.downloadFile(localUrl, filePath);
            return localUrl;
          }

          if (file.fileSize == 0 || file.lastModifiedTime + ttl < now)
          {
            if ( file.isWritable() )
              file.remove(false);
            return localUrl;
          }
          return "file:///" + filePath;
        } else
          return localUrl;
      },
      updateButtons: function(data)
      {
        var sandbox = Components.utils.Sandbox("about:blank");
        sandbox.listeners = pageturn_listeners;
        sandbox.data = data;
        sandbox.trace = trace;
        Components.utils.evalInSandbox("for (var i = 0; i < listeners.length; i++) { listeners[i](data) }", sandbox);

        for (var i = 0; i < toolbar_headlines.length; i++)
        {
          loadContentHelp.getRightColor(toolbar_headlines[i]);
        }
        
        for (key in toolbar_buttons)
        {
          let child = toolbar_buttons[key].child;
          let properties = toolbar_buttons[key].properties;

          if ( ("AMAZON_NS_PH" != "ALX_NS_PH") && (properties.type != "search") )
              loadContentHelp.getRightColor(child);
          if ( ("AMAZON_NS_PH" == "ALX_NS_PH") && (properties.type != "search" && properties.type == "javascript") )
              loadContentHelp.getRightColor(child);

          if (properties.image)
          {
            if (properties.updated && properties.updatedImage)
            {
              child.image = ALX_NS_PH_TB.checkLocalDefault( properties.updatedImage );
            }
            else
            {              
              child.image = ALX_NS_PH_TB.checkLocalDefault( properties.image );
            }
            if (properties.hoverImage)
            {
              let image = ALX_NS_PH_TB.checkLocalDefault( properties.image ),
                  hoverImage = ALX_NS_PH_TB.checkLocalDefault( properties.hoverImage ),
                  updatedImage = ALX_NS_PH_TB.checkLocalDefault( properties.updatedImage ),
                  updatedHoverImage = ALX_NS_PH_TB.checkLocalDefault( properties.updatedHoverImage );
              
              child.onmouseover = function()
              {
                if (properties.updated && updatedHoverImage)
                {
                  child.image = updatedHoverImage;
                }
                else if (properties.updated && updatedImage)
                {
                  child.image = updatedImage;
                }
                else
                {
                  child.image = hoverImage;
                }
              }
              child.onmouseout = function()
              {
                if (properties.updated && updatedImage)
                {
                  child.image = updatedImage;
                }
                else
                {
                  child.image = image;
                }
              }
            }
          }
          if (properties.tooltip)
            child.setAttribute("tooltiptext", subst_tokens(properties.tooltip, data));
          if (properties.text)
            child.setAttribute("label", subst_tokens(properties.text, data));
          child.removeEventListener("command", child.oncommand, false);
          child.removeAttribute("oncommand");
          if (properties._oncommand)
          {
            child.oncommand = properties._oncommand;
            child.addEventListener("command", child.oncommand, false);
          }
          else if (properties.url && properties.url != "about:blank")
          {
            child.oncommand = function(event)
            {
              var openIn = "currentTab";
              var windowFeatures = "";
              var url = properties.url;
              if (properties.openIn)
                openIn = properties.openIn;
              if (properties.windowFeatures)
                windowFeatures = properties.windowFeatures;
              ALX_NS_PH_TB.log(properties.id + ':c');
              ALX_NS_PH_TB.loadURL(event, properties.url, openIn, windowFeatures);
            }
            child.addEventListener("command", child.oncommand, false);
          } else if (child.getAttribute("type") == 'menu-button')
          {
            child.setAttribute("type", "menu");
            /*child.oncommand = function(event)
            {
              var popup = child.childNodes[0];
              popup.openPopup(child, "after_start", 0, 0, false, false);
              ALX_NS_PH_TB.log(properties.id + ':c');
              //ALX_NS_PH_TB.loadURL(event, properties.url, openIn, windowFeatures);
            }
            child.addEventListener("command", child.oncommand, false);*/
          }
        }
        this.updatePopups();
      },
      loadSearchURL: function(event, url, openIn, id) {
        if (!id)
          id = "ALX_NS_PH_TB-Search-Edit";
        var searchTermsBox = document.getElementById(id);
        
        if (searchboxes[id]["isDefaultText"] == true)
          return;
        var searchTerms = trim(searchTermsBox.value);
        if (searchTerms == "")
        {
          searchTermsBox.value = "";
          return;
        }
        searchTerms = encodeURIComponent(searchTerms);
        try {
          url = url.replace(/__TERM__PLACEHOLDER__/g, searchTerms);
        } catch (ex) {
          try {
            url = url.replace(/__TERM__PLACEHOLDER__/g, "");
          } catch (ex) {

          }
        }
        var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
        try {
          var session = prefBranch.getCharPref("extensions.alexa.session");
          url = url.replace(/__AID__PLACEHOLDER__/g, session);
        }
        catch (ex)
        {
          url = url.replace(/__AID__PLACEHOLDER__/g, "");
        }

        return ALX_NS_PH_TB.internalLoadURL(url, openIn);
      },
      register_buttons: function(toolbar_id, url, skip_confirm) {
        var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        var http = new XMLHttpRequest();
        http.open( "GET", url, true );
        http.onload = function()
        {
          var xml = http.responseXML;
          var id = xml.getElementsByTagName("id")[0].textContent;
          var title = xml.getElementsByTagName("title")[0].textContent;
          var minversion = xml.getElementsByTagName("ff-minversion");
          var version = alexaUtilityService.getMyVersion();
          if (minversion && minversion[0] && minversion[0].textContent && verSvc.compare(minversion[0].textContent, version) > 0)
          {
            if (!skip_confirm)
            {
              promptSvc.alert(null, "Upgrade your toolbar", "This button requires a newer version of the toolbar. Upgrade your toolbar now.");
            }
            prefBranch.setCharPref("extensions.alexa.active-buttons." + id + ".version", alexaUtilityService.getMyVersion());
            return;
          }
          var text = http.responseText;
          var active_buttons = prefBranch.getCharPref("extensions.alexa.active-buttons").split(",");
          if (id && (skip_confirm || promptSvc.confirm(null, "Install Button", "Would you like to add the " + title + " button?")))
          {
            prefBranch.setCharPref("extensions.alexa.active-buttons." + id + ".url", url);
            prefBranch.setCharPref("extensions.alexa.active-buttons." + id + ".version", alexaUtilityService.getMyVersion());
            prefBranch.setCharPref("extensions.alexa.active-buttons." + id, text);
            if (active_buttons.indexOf(id) == -1)
            {
              ALX_NS_PH_TB.log(id + ':n');
              active_buttons.push(id);
              prefBranch.setCharPref("extensions.alexa.active-buttons", active_buttons.join(","));
            }
            else
            {
              ALX_NS_PH_TB.extension.init();
            }
          }
        };
        http.send(null);
      },
      log: function(name)
      {
        if (events.indexOf(name) == -1)
        {
          events.push(name);
        }
      },
      events: function()
      {
        return events.join(";");
      },
      clear_events: function()
      {
        events.length = 0;
      }
    }
  } // ALX_NS_PH_TB
  if (typeof ALX_NS_PH_TB.rss == "undefined") {
    function shuffle(oldArray) {
      var myArray = [];
      for (var i = 0; i < oldArray.length; i++)
      {
        myArray.push(oldArray[i]);
      }
      if ( i == 0 ) return false;
      while ( --i ) {
        var j = Math.floor( Math.random() * ( i + 1 ) );
        var tempi = myArray[i];
        var tempj = myArray[j];
        myArray[i] = tempj;
        myArray[j] = tempi;
      }
      return myArray;
    }

    ALX_NS_PH_TB.rss = {
      initRSSButton: function(toolbaritem, toolbarchild, properties, menupopup, refreshFunc)
      {
        var id = properties.id;
        var buttonId = "ALX_NS_PH_TB-RSS-" + id + "-Menu"
        menupopup.setAttribute("id", buttonId); 
        menupopup.addEventListener("popupshowing", function(event)
        {
          ALX_NS_PH_TB.log(id + ":m");
          if (properties.updated)
          {
            properties.updated = false;
            ALX_NS_PH_TB.updateButtons(get_data());
          }
        }, false);
        toolbarchild.appendChild(menupopup);
  
        // leave the place holder for the rss headline
        var numHeadlines = properties.numHeadlines;
        for (var i = 0; i < numHeadlines; i++)
        {
          if (i > 0)
          {
            var interpunct = document.createElement("image");
            interpunct.setAttribute("src", "chrome://ALX_NS_PH/skin/middot.png");
            interpunct.setAttribute("height", "3");
            interpunct.setAttribute("width", "3");
            interpunct.setAttribute("style", "margin-left: 1px; margin-right: 1px");
            interpunct.setAttribute("id", "ALX_NS_PH_TB-RSS-" + properties.id + "-Button" + i + "-interpunct"); 
            toolbaritem.appendChild(interpunct);
          }
          var headline = document.createElement("toolbarbutton");
          headline.setAttribute("id", "ALX_NS_PH_TB-RSS-" + properties.id + "-Button" + i); 
          headline.setAttribute("class", "ALX_NS_PH_TB_headline"); 
          headline.setAttribute("style", "cursor: pointer; padding-left: 2px !important; padding-right: 2px !important; /*-moz-binding: url(chrome://ALX_NS_PH/content/button.xml#alexatoolbarbutton)*/");
          toolbar_headlines.push(headline);
          toolbaritem.appendChild(headline);
        }
        if (numHeadlines)
        {
          toolbaritem.setAttribute("flex", "1");
          loadContentHelp.addSepar(toolbaritem);
        }
        
        // get the fillin function
        //var fillRSSButton = refreshFunc;
        
        // add and register refresh button
        var menuitem = document.createElement("menuitem");
        if (properties.id != "amazontweet")
        {
          if (properties.refreshText)
            menuitem.setAttribute("label", properties.refreshText);
          else
            menuitem.setAttribute("label", "Refresh");
          menuitem.oncommand = function(event)
          {
            ALX_NS_PH_TB.log(id + ':i');
            refreshFunc(properties, true);
            event.stopPropagation();
          };
        } else {
          if (properties.nothingAvailable)
            menuitem.setAttribute("label", properties.nothingAvailable);
          else
            menuitem.setAttribute("label", "No data Available");
          menuitem.style.color = "gray";
          menuitem.oncommand = function(event)
          {
            event.stopPropagation();
          };
        }

        menuitem.addEventListener("command", menuitem.oncommand, false);
        menupopup.appendChild(menuitem);
  
      },
      
      fillRSSButton: function(properties, nocache)
      {
        if (!feedwrappers[properties.id])
        {
          feedwrappers[properties.id] = new feedWrapper(properties);
        }
        else
        {
          feedwrappers[properties.id].properties = properties;
        }
        feedwrappers[properties.id].loadRss(nocache);
      }
    }
  }
  if (typeof ALX_NS_PH_TB.extension == "undefined") {
    ALX_NS_PH_TB.extension = {
      collapsedBoundary: null,
      getCurrentDocument: function() {
        return gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentWindow.document;
      },
      resize: function() {
        var toolbarspring = document.getElementById("ALX_NS_PH_TB-Toolbar-Spring");
        var toolbar = document.getElementById("ALX_NS_PH_TB-Toolbar");
        var lastRight = toolbar.lastChild;
        var rightBoundary = window.innerWidth;
        var lastRightBoundingBox = lastRight.getBoundingClientRect();
        if (lastRightBoundingBox)
          var leftBoundary = lastRightBoundingBox["right"];
        else
          var leftBoundary = rightBoundary;

        if (rightBoundary < 400)
          return;

        var collapsedList = [];
        if (leftBoundary > rightBoundary) // shrinking
        {
          var currentNode = null;
          var hideItems = null;
          var hideItemsPopupShow = null;

          if (ALX_NS_PH_TB.extension.collapsedBoundary)
            currentNode = ALX_NS_PH_TB.extension.collapsedBoundary.previousSibling;
          else
            currentNode = toolbarspring.previousSibling;

          var currentBox = currentNode.getBoundingClientRect();
          var currentRight = currentBox["right"];
          var currentWidth = currentRight - currentBox["left"];
          while ( (leftBoundary - rightBoundary) > currentWidth )
          {
            currentNode.setAttribute("previous", currentBox["right"] - currentBox["left"]);
            ALX_NS_PH_TB.extension.collapsedBoundary = currentNode;
            collapsedList.push(currentNode);
            currentNode = currentNode.previousSibling;
            currentBox = currentNode.getBoundingClientRect();
            currentWidth = currentRight - currentBox["left"];
          }
          currentNode.setAttribute("previous", currentBox["right"] - currentBox["left"]);
          ALX_NS_PH_TB.extension.collapsedBoundary = currentNode;
          collapsedList.push(currentNode);

          if (ALX_NS_PH_TB.extension.collapsedBoundary)
          {
            hideItems = document.getElementById("ALX_NS_PH_TB-Toolbar-HideItms");
            if (!hideItems)
            {
              hideItems = document.createElement("toolbarbutton");
              hideItems.setAttribute("id", "ALX_NS_PH_TB-Toolbar-HideItms");
              hideItems.setAttribute("image", "chrome://ALX_NS_PH/skin/hb.ico");
              toolbar.insertBefore(hideItems, toolbarspring.nextSibling);
              hideItemsPopupShow = document.createElement("menupopup");
              hideItemsPopupShow.setAttribute("id", "ALX_NS_PH_TB-Toolbar-HideItms-MenuPopup");
              hideItems.appendChild(hideItemsPopupShow);
              /*var menuItem = document.createElement("menuitem");
              hideItemsPopupShow.appendChild(menuItem);*/
              hideItems.addEventListener("command", function () {
                hideItemsPopupShow.openPopup(hideItems, "after_end", 0, 0, true, false);
              }, false);
            }
          }

          for (var i = 0; i < collapsedList.length; ++i)
          {
            var node = collapsedList[i];
            node.setAttribute("collapsed", "true");
            var tagName = node.tagName;

            if (tagName == "toolbaritem")
            {
              var button = node.firstChild;
              var id = node.getAttribute("id").replace(/ALX_NS_PH_TB-Item-/gi, "");
              tagName = button.tagName;
              if (tagName == "toolbarbutton")
              {
                var image = button.getAttribute("image");
                var tooltip = button.getAttribute("tooltiptext");
                var menuItem = document.createElement("menuitem");

                if (toolbar_buttons[id].properties.shortText)
                  menuItem.setAttribute("label", subst_tokens(toolbar_buttons[id].properties.shortText, get_data()));

                menuItem.setAttribute("id", "ALX_NS_PH_TB-Toolbar-HideItms-MenuItem-" + id);
                menuItem.setAttribute("image", image);
                menuItem.setAttribute("tooltiptext", subst_tokens(tooltip));
                menuItem.setAttribute("class", "menuitem-iconic alexa-favicon");

                if (button.oncommand)
                  menuItem.addEventListener("command", button.oncommand, false);
                
                if (!hideItemsPopupShow)
                  hideItemsPopupShow = document.getElementById("ALX_NS_PH_TB-Toolbar-HideItms-MenuPopup");
                hideItemsPopupShow.appendChild(menuItem);

              }
            }
          }

          return;
        }

        var toolbarspringBoundingBox = toolbarspring.getBoundingClientRect();
        var toolbarspringWidth = toolbarspringBoundingBox["right"] - toolbarspringBoundingBox["left"];

        if (ALX_NS_PH_TB.extension.collapsedBoundary && toolbarspringWidth > 38) // expanding
        {
          var previousWidth = parseInt(ALX_NS_PH_TB.extension.collapsedBoundary.getAttribute("previous"), 10);
          while (ALX_NS_PH_TB.extension.collapsedBoundary && ( previousWidth < (toolbarspringWidth - 38)) )
          {
            collapsedList.push(ALX_NS_PH_TB.extension.collapsedBoundary);
            ALX_NS_PH_TB.extension.collapsedBoundary = ALX_NS_PH_TB.extension.collapsedBoundary.nextSibling;
            toolbarspringWidth = toolbarspringWidth - previousWidth - 8;
            if (ALX_NS_PH_TB.extension.collapsedBoundary == toolbarspring)
            {
              ALX_NS_PH_TB.extension.collapsedBoundary = null;
            } else
            {
              previousWidth = parseInt(ALX_NS_PH_TB.extension.collapsedBoundary.getAttribute("previous"), 10);
            }
          }

          for (var i = 0; i < collapsedList.length; ++i)
          {
            var node = collapsedList[i];
            node.setAttribute("collapsed", "");
            node.removeAttribute("previous");
            try {
              var id = node.getAttribute("id").replace(/ALX_NS_PH_TB-Item-/gi, "");
              var hideNode = document.getElementById("ALX_NS_PH_TB-Toolbar-HideItms-MenuItem-" + id);
              var popupNode =  document.getElementById("ALX_NS_PH_TB-Toolbar-HideItms-MenuPopup");
              popupNode.removeChild(hideNode);
            } catch(e) {}

          }

          if (ALX_NS_PH_TB.extension.collapsedBoundary == null)
          {
            var hideItems = document.getElementById("ALX_NS_PH_TB-Toolbar-HideItms");
            if (hideItems)
            {
              toolbar.removeChild(hideItems);
            }
          }

          return;
        }
      },
      fullscreen: function() {
        //trace("fullscreen on or off: " + window.fullScreen);
        if ( !window.fullScreen )
        {
          for (var i in popup_panels)
          {
            if (popup_panels[i] && !popup_panels[i].closed)
            {
              popup_panels[i].close();
            }
          }
        } 
      },
      init: function() {      
        if (toolbar.visible)
        {
          var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
          loadContentHelp.currentLocale = "default";
          if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocale") )
            loadContentHelp.currentLocale = prefBranch.getCharPref("extensions.alexa.toolbarLocale");

          if (init_in_progress)
          {
            return;
          }
          init_in_progress = true;
          init_toolbar();

          var branch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
          var observer = {
            observe: function(subject, topic, data)
            {
              if (topic == "nsPref:changed")
              {
                if (data == "extensions.alexa.requestUninstall")
                {
                  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   . getService(Components.interfaces.nsIPrefService);
                  var reString = prefs.getCharPref("extensions.alexa.requestUninstall");
                  var session  = prefs.getCharPref("extensions.alexa.session");
                  var reasons = [reString, "aid=" + session, "ver=" + alexaUtilityService.getMyVersion()];
                  
                  var logurl = "http://log.alexa.com/uninstall/survey/?" + reasons.join("&");
                  var http = new XMLHttpRequest();
                  http.open( "HEAD", logurl, true );
                  http.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
                  http.channel.loadFlags |= Ci.nsIRequest.LOAD_BACKGROUND;
                  http.channel.loadGroup = requestor.getInterface(Ci.nsILoadGroup);
                  http.send( null );

                  // Notify the toolbar that it is being uninstalled
                  prefs.setBoolPref("uninstall", true);
                  Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("general.useragent.extra.").deleteBranch("ALX_NS_PH");
                  return;
                }

                if (data == "extensions.alexa.toolbarLocaleText")
                {
                  loadContentHelp.stringBundle = null;
                  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   . getService(Components.interfaces.nsIPrefService);
                  var rssbranch = prefs.getBranch("extensions.alexa.active-buttons.");
                    
                  for (var key in toolbar_buttons)
                  {
                    var properties = toolbar_buttons[key].properties
                    if (properties.type && properties.type == "rss")
                    {
                      var rsskey = key + '.rss.items';
                      if (rssbranch.prefHasUserValue(rsskey))
                        rssbranch.setCharPref(rsskey, "");
                    }
                  }
                } 

                init_toolbar();
              }
            },
            shutdown: function()
            {
              branch.removeObserver("extensions.alexa.toolbarXMLText", this);
              branch.removeObserver("extensions.alexa.toolbarLocaleText", this);
            }
          };
          branch.addObserver("extensions.alexa.toolbarXMLText", observer, false);
          branch.addObserver("extensions.alexa.toolbarLocaleText", observer, false);
          window.addEventListener("unload",
                      function() {observer.shutdown();},
                      false);
        }
      },
      shutdown: function() {
        logo = null;
        lastAmazonpopup = null;
        var toolbar = document.getElementById("ALX_NS_PH_TB-Toolbar");
        while (toolbar.firstChild) {
          toolbar.removeChild(toolbar.firstChild);
        }
        toolbar_buttons = {};
        toolbar_headlines.length = 0;
        for (var i = 0; i < timers.length; i++)
        {
          timers[i].cancel();
        }
        timers.length = 0;
        for (var i in popup_panels)
        {
          if (popup_panels[i])
          {
            popup_panels[i].close();
          }
          delete popup_panels[i];
        }
        for (var i in iframes)
        {
          if (iframes[i])
          {
            iframes[i].close();
          }
          delete iframes[i];
        }
        for (var i in feedwrappers)
        {
          feedwrappers[i].shutdown();
          delete feedwrappers[i];
        }
        for (var i in observers)
        {
          observers[i].shutdown();
          delete observers[i];
        }
      },
    }
  } // ALX_NS_PH_TB.extension
  if (typeof ALX_NS_PH_TB.eventHandlers == "undefined") {
    ALX_NS_PH_TB.eventHandlers = {
      keyHandler: function(event) {
        // Was the key that was pressed [ENTER]? If so, perform a web search.
        if (event.keyCode == event.DOM_VK_RETURN) {
          ALX_NS_PH_TB.loadSearchURL(event, 0);
        }
      }
    }
  }

})();

window.addEventListener("load",
            function() {ALX_NS_PH_TB.extension.init();},
            false);
window.addEventListener("unload",
            function() {ALX_NS_PH_TB.extension.shutdown();},
            false);
window.addEventListener("fullscreen",
            function() {ALX_NS_PH_TB.extension.fullscreen();},
            false);
//window.addEventListener("resize",
//            function() {ALX_NS_PH_TB.extension.resize();},
//            false);

window.addEventListener("ALX_NS_PH_EVENT",
  function(evt)
  {
    var localUrl = evt.target.ownerDocument.documentURI;
    if ( localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar") == 0 || localUrl.indexOf("https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/buttons") == 0 )
    {
      switch(evt.target.getAttribute("function"))
      {
        case "register_buttons":
          ALX_NS_PH_TB.register_buttons(
            "ALX_NS_PH", evt.target.getAttribute("url"), false
          );
          break;
        case "configure":
          var params = evt.target.getAttribute("params");
          var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
          params = nativeJSON.decode(params);
          ALX_NS_PH_TB.configure(params);
          break;
        case "set_locale":
          var locale = evt.target.getAttribute("locale");
          var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
          prefBranch.setCharPref("extensions.alexa.toolbarLocaleChange", locale);
          break;
        default:
          window.alert(evt.target.getAttribute("function"));
          break;
      }
    }
  },
  false, true
);
