const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;
const MY_CID = "{65963fbb-56b1-435f-88b1-fa20287cb80d}";
var urlService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
var dnsService = Cc["@mozilla.org/network/dns-service;1"].getService(Ci.nsIDNSService);
var EQS = Ci.nsIEventQueueService;
var regex = /^(0|10|127|172\.(1[6-9]|2[0-9]|3[0-1])|169\.254|192\.168|255)\./;
var prefs = Components.classes["@mozilla.org/preferences-service;1"]
          . getService(Components.interfaces.nsIPrefService)
          ;
var branch = prefs.getBranch("");
var nextPingTime = 0;
var need_data = false;
var pinger = null;

var shell = Cc["@mozilla.org/docshell;1"];
if (!shell)
{
  // FF 3.5 and earlier
  shell = Cc["@mozilla.org/webshell;1"];
}
var requestor = shell.createInstance(Ci.nsIDocShellTreeItem).QueryInterface(Ci.nsIInterfaceRequestor)

function trace(msg)
{
  Cc["@mozilla.org/consoleservice;1"]
    .getService(Ci.nsIConsoleService)
    .logStringMessage("<"+String(msg)+">");
}

function myComponent() { }

myComponent.prototype = {
  baseline: "http://data.alexa.com/data",
  sessionRequested: false,
  QueryInterface: function (iid) {
    if (iid.equals(Ci.nsISupports) || iid.equals(Ci.nsIAlexaUtilityService) || MY_CID.equals(iid))
      return this;

    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;

    return null;
  },
  checkIntranet: function(url, callback)
  {
    if (urlService.offline)
    {
      callback.callback(url, true);
      return;
    }
    var thread;
    if (Cc["@mozilla.org/event-queue-service;1"]) {
      thread = Cc["@mozilla.org/event-queue-service;1"].getService(EQS).getSpecialEventQueue(EQS.CURRENT_THREAD_EVENT_QUEUE);
    } else {
      thread = Cc["@mozilla.org/thread-manager;1"].getService().mainThread;
    }
    dnsService.asyncResolve(url.host, 0, {
      onLookupComplete: function(request, result, status) {
        try
        {
          while ( result && result.hasMore() )
          {
            var ip = result.getNextAddrAsString();
            if ( ip && ip.match( regex  ) )
              return callback.callback(url, true);
          }
        }
        catch (ex)
        {
          return callback.callback(url, true);
        }

        return callback.callback(url, false);
      }
    }, thread);
  },
  getMyVersion: function()
  {
    try
    {
      return "alxf-" + "2.01";
    } catch(ex) { return "alxf"; }
  },
  requestSession: function(browser)
  {
    if (this.sessionRequested)
      return;
    var prefBranch2 = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
    prefBranch2.addObserver("extensions.alexa.", singleton, false);
    prefBranch2.addObserver("general.useragent.locale", singleton, false);
    prefBranch2.addObserver("extensions.alexa.toolbarLocaleChange", singleton, false);
    this.sessionRequested = true;
    var pref = "extensions.alexa.session";
    var version = this.getMyVersion();
    if ( !branch.prefHasUserValue(pref) || branch.getCharPref(pref).length == 0 )
    {
      try
      {
        var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
        http.open( "GET", this.baseline + "?cli=10&stc", true );
        var self = this;
        http.onload = function ()
        {
          try
          {
            if ( "OK" == http.statusText )
            {
              var session = http.responseXML
                                .documentElement
                                .attributes
                                .getNamedItem("AID")
                                .value;
              if (session && session != "=")
              {
                branch.setCharPref( pref, session );
              }
            }
          }
          catch(ex)
          {
          }
        }
        http.send( null );
      }
      catch(ex)
      {
      }
    }
    else
    {
      var aid = branch.getCharPref(pref);      
      this.init_pinger();
    }
  },
  observe: function(subject, topic, data)
  {
    if (topic == "nsPref:changed")
    {
      // Log enabling and disabling of features
      var params = [
        "name=" + encodeURIComponent(data),
      ];

      try
      {
        switch (branch.getPrefType(data))
        {
          case branch.PREF_BOOL:
            params.push("value=" + (branch.getBoolPref(data) ? "1" : "0"));
            break;
          case branch.PREF_STRING:
            params.push("value=" + branch.getCharPref(data));
            break;
          case branch.PREF_INT:
            params.push("value=" + branch.getIntPref(data));
            break;
          default:
            break;
        }
      }
      catch (ex)
      {
        trace(ex);
      }

      // Don't log button-specific data, which might contain auth credentials
      if (data && -1 == data.indexOf(".active-buttons.") && -1 == data.indexOf(".signal."))
      {
        this.log("prefs", params.join("&"));
      }

      // change the locale text
      if ( data == "extensions.alexa.toolbarLocaleChange" )
      {
        var locale = branch.getCharPref(data);
        locale = locale.replace(/\s+/gi, "");
        if (locale)
        {
          this.getLocaleXml( true, true, locale);
          branch.setCharPref(data, "");    
        }
      }
      if ( data == "general.useragent.locale" )
      {
        this.getLocaleXml( false, true );
      }

      var aid = "";
      if (branch.prefHasUserValue("extensions.alexa.session"))
      {
        aid = branch.getCharPref("extensions.alexa.session");
      }
      if (data == "extensions.alexa.location-bar-search" ||
          data == "extensions.alexa.disabled" ||
          data == "extensions.alexa.uninstall" ||
          data == "extensions.alexa.session" ||
          data == "extensions.alexa.configured2")
      {
        this.configure_keyword_search(aid);
      }

      if (!pinger && aid)
      {
        this.init_pinger();
      }

      var needed_data = need_data;
      need_data = this.needDataRequests();
      if (needed_data != need_data)
      {
        this.log("measure", "enabled=" + (need_data ? "1" : "0"));
      }
    }
  },
  log: function(prefix, extra)
  {
    var aid = "";
    if (branch.prefHasUserValue("extensions.alexa.session"))
    {
      aid = branch.getCharPref("extensions.alexa.session");
    }
    var params = [
      "aid=" + encodeURIComponent(aid),
      "plugin=" + encodeURIComponent(this.getMyVersion()),
      extra
    ];
    var logurl = "http://log.alexa.com/" + prefix + "/?" + params.join("&");
    var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    http.open( "HEAD", logurl, true );
    http.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
    http.channel.loadFlags |= Ci.nsIRequest.LOAD_BACKGROUND;
    http.channel.loadGroup = requestor.getInterface(Ci.nsILoadGroup);
    http.send( null );
  },
  configure_keyword_search: function(aid)
  {
    var searchService = Cc["@mozilla.org/browser/search-service;1"]
                        .getService(Ci.nsIBrowserSearchService);
    var name = "Web Search (powered by Google)";
    var ver = this.getMyVersion( );
    var url = "http://search.toolbars.alexa.com/?ver=" + encodeURIComponent(ver);
    if (branch.getBoolPref('extensions.alexa.location-bar-search') &&
        !branch.getBoolPref("extensions.alexa.disabled") &&
        !branch.getBoolPref("extensions.alexa.uninstall"))
    {
      branch.setCharPref("keyword.URL", url + "&src=ab&aid=" + encodeURIComponent(aid) + "&q=");
      var toolbarSearch = searchService.getEngineByName(name);
      if (!toolbarSearch)
      {
        searchService.addEngineWithDetails(name, "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/search-icon.png", "",
                                           name, "GET", url + "&src=bc&aid=" + encodeURIComponent(aid) + "&q={searchTerms}");
        toolbarSearch = searchService.getEngineByName(name);
      }
      searchService.currentEngine = toolbarSearch;
    }
    else
    {
      if (branch.prefHasUserValue('keyword.URL'))
      {
        branch.clearUserPref("keyword.URL");
      }
      var toolbarSearch = searchService.getEngineByName(name);
      if (toolbarSearch)
      {
        searchService.removeEngine(toolbarSearch);
        if (!branch.getBoolPref("extensions.alexa.disabled") && !branch.getBoolPref("extensions.alexa.uninstall"))
        {
          try {
            searchService.addEngineWithDetails(name, "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/search-icon.png", "",
                                               name, "GET", url + "&src=bc&aid=" + encodeURIComponent(aid) + "&q={searchTerms}");
            var firstengine = searchService.getEngines({})[0];
            searchService.currentEngine = firstengine;
          } catch(e) { trace(e) ;}
        }
      }
    }
  },
  init_pinger: function()
  {
    if (!pinger)
    {
      var PING_DELAY = 86400000;
      pinger = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      pinger.initWithCallback({
        notify: function()
        {
          singleton.dailyJob()
        }
      }, PING_DELAY, Ci.nsITimer.TYPE_REPEATING_SLACK);
      this.dailyJob();
    }
  },
  dailyJob: function()
  {
    this.getGIURls();
    this.getToolbarXml();
    this.getLocaleXml( false, false );
    this.ping();
  },
  getGIURls: function()
  {
    var ver = this.getMyVersion();
    var src = "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/giurls.js";
    var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    http.open( "GET", src, true );
    http.onload = function()
    {
      if (http.status == 200)
      {

        var contentText = http.responseText;
        var contentXml  = http.responseXML;

        var error = [];
        if (contentXml)
          error = contentXml.getElementsByTagName("Error");

        if (contentText && error.length == 0)
        {
          var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
          var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON);
          var giURlsObject = nativeJSON.decode("(" + contentText + ")");
          if (giURlsObject.giResponseTimeOut)
            prefBranch.setIntPref("extensions.alexa.giurls.giResponseTimeOut", giURlsObject.giResponseTimeOut);
          if (giURlsObject.giUrls)
          {
            prefBranch.setCharPref("extensions.alexa.giurls.giUrls", nativeJSON.encode(giURlsObject.giUrls));
          }
        }
      }
    };
    http.send(null);
  },
  getToolbarXml: function()
  {
    var ver = this.getMyVersion();
    var src = "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/toolbar.xml";
    //trace("src:" + src);
    var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    http.open( "GET", src, true );
    http.onload = function()
    {
      //trace("Status: " + http.status);
      if (http.status == 200)
      {
        
        var contentText = http.responseText;
        var contentXml  = http.responseXML;
        
        var error = contentXml.getElementsByTagName("Error");
        var button = contentXml.getElementsByTagName("button");
        
        if (contentText && error.length == 0 && button.length > 0)
        {
          var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
          prefBranch.setCharPref("extensions.alexa.toolbarXMLVersion", ver);
          prefBranch.setCharPref("extensions.alexa.toolbarXMLText", unescape(encodeURIComponent(contentText)));

          var tmpText = null;
          if (prefBranch.prefHasUserValue("extensions.alexa.toolbarXMLText"))
            tmpText = prefBranch.getCharPref("extensions.alexa.toolbarXMLText");
            
          contentText = unescape(encodeURIComponent(contentText));
          if (tmpText != contentText)
            prefBranch.setCharPref("extensions.alexa.toolbarXMLText", contentText);

          //prefBranch.setCharPref("extensions.alexa.toolbarXMLText", unescape(encodeURIComponent(contentText)));
        }
      }
    };
    http.send(null);
  },
  getLocaleXml: function( isLangOnly, isForce, changedLocale )
  {
    var func = this.getLocaleXml;
    var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
    var http = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    var locale = null;
    var isToolbarLocale = null; 

    if ( changedLocale )
    {
      locale = changedLocale; 
      isToolbarLocale = true;
    }
    else if ( !isForce && prefBranch.prefHasUserValue("extensions.alexa.toolbarLocale") )
    {
      locale = prefBranch.getCharPref("extensions.alexa.toolbarLocale");
      isToolbarLocale = true;
    } else { 
      locale = prefBranch.getCharPref("general.useragent.locale").toLowerCase();
      if (isLangOnly)
      {
        var tmpArray = locale.split("-");
        locale = tmpArray[0];
      }
      isToolbarLocale = false;
    }
    var src = "https://s3.amazonaws.com/com.alexa.toolbar/alx/f/2.01/toolbar/locale/" + locale + "/locale_resources.xml";
    
    http.open( "GET", src, true );
    http.onload = function()
    {
      var toolbarLocaleText = null;
      if (http.status == 200)
      {
        var contentText = http.responseText;
        var contentXml  = http.responseXML;
        
        var error = contentXml.getElementsByTagName("Error");
        var button = contentXml.getElementsByTagName("button");
        
        if (contentText && error.length == 0 && button.length > 0)
        {
          prefBranch.setCharPref("extensions.alexa.toolbarLocale", locale);
          toolbarLocaleText = unescape(encodeURIComponent(contentText));
        }
      }
      
      if (toolbarLocaleText == null && isLangOnly)
      {
        prefBranch.setCharPref("extensions.alexa.toolbarLocale", "default");
        toolbarLocaleText = "";
      }
      if ( toolbarLocaleText != null )
      {
        var tmpText = null;
        if ( prefBranch.prefHasUserValue("extensions.alexa.toolbarLocaleText") )
          tmpText = prefBranch.getCharPref("extensions.alexa.toolbarLocaleText");
        if(tmpText != toolbarLocaleText)
          prefBranch.setCharPref("extensions.alexa.toolbarLocaleText", toolbarLocaleText);
        return;
      }
      
      if (isToolbarLocale == false)
        func( true , isForce);
    };
    http.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
    http.channel.loadFlags |= Ci.nsIRequest.LOAD_BACKGROUND;
    http.channel.loadGroup = requestor.getInterface(Ci.nsILoadGroup);
    http.send(null);
  },
  ping: function( isNew )
  {
    var searchService = Cc["@mozilla.org/browser/search-service;1"]
                        .getService(Ci.nsIBrowserSearchService);
    var name = "Web Search (powered by Google)";
    var toolbarSearch = searchService.getEngineByName(name);
    var isErrorHandling = false;
    var errorConfName = "extensions.alexa.error-handling";
    if ( branch.getBoolPref( errorConfName ) )
      isErrorHandling = true;

    var params = [
      "dad=" + ( this.needDataRequests() ? "1" : "0" ),
      "search_provider=" + (searchService.currentEngine == toolbarSearch ? "1" : "0"),
      "error_handling=" + (isErrorHandling ? "1" : "0") 
    ]
    
    if (isNew)
      params.push("new=1");

    this.log("ping", params.join("&"));
  },
  needDataRequests: function()
  {
    return (branch.getBoolPref("extensions.alexa.ranks") ||
            branch.getBoolPref("extensions.alexa.alexa-relatedlinks") ||
            branch.getBoolPref("extensions.alexa.reviews"));
  }
};

var singleton = null;
var createInstanceFunc = function (outer, iid) {
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;
  
  if (!singleton)
  {
    singleton = (new myComponent());
    need_data = singleton.needDataRequests();
  }

  return singleton.QueryInterface(iid);
};

var myModule = {
  firstTime: true,

  /*
   * RegisterSelf is called at registration time (component installation
   * or the only-until-release startup autoregistration) and is responsible
   * for notifying the component manager of all components implemented in
   * this module.  The fileSpec, location and type parameters are mostly
   * opaque, and should be passed on to the registerComponent call
   * unmolested.
   */
  registerSelf: function (compMgr, fileSpec, location, type) {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID,
                                    "Alexa Utility Service",
                                    this.myProgID,
                                    fileSpec,
                                    location,
                                    type);
  },

  unregisterSelf: function(compMgr, fileSpec, location) {
    var compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
    compMgr.unregisterFactoryLocation(this.myCID, fileSpec);
  },

  /*
   * The GetClassObject method is responsible for producing Factory objects
   */
  getClassObject: function (compMgr, cid, iid) {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    if (!iid.equals(Ci.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.myFactory;
  },

  /* CID for this class */
  myCID: Components.ID(MY_CID),

  /* ProgID for this class */
  myProgID: "@ALX_NS_PH/utility-service;1",

  /* factory object */
  myFactory: {
    /*
     * Construct an instance of the interface specified by iid, possibly
     * aggregating it with the provided outer.  (If you don't know what
     * aggregation is all about, you don't need to.  It reduces even the
     * mightiest of XPCOM warriors to snivelling cowards.)
     */
    createInstance: createInstanceFunc
  },

  createInstance: createInstanceFunc,

  /*
   * The canUnload method signals that the component is about to be unloaded.
   * C++ components can return false to indicate that they don't wish to be
   * unloaded, but the return value from JS components' canUnload is ignored:
   * mark-and-sweep will keep everything around until it's no longer in use,
   * making unconditional ``unload'' safe.
   *
   * You still need to provide a (likely useless) canUnload method, though:
   * it's part of the nsIModule interface contract, and the JS loader _will_
   * call it.
   */
  canUnload: function(compMgr) {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  return myModule;
}

function NSGetFactory(cid) {
    if (cid.toString().toUpperCase() != MY_CID.toString().toUpperCase()) throw Components.results.NS_ERROR_FACTORY_NOT_REGISTERED;
    return myModule;
}
